(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa/isa_toolkit/tagging/tag.sml,v $
 
   Selection in Isabelle terms
 
   $Date: 1997/09/14 14:50:33 $
   $Revision: 1.11 $
   Author: B. Wolff, C. Lueth (Last modification by $Author: bu $)

   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
 ************************************************************************** *)


structure TermSel :
  sig
    exception IllegalPath of string

    type path  (* a path into a term *)

    val encode : path   -> string
    val decode : string -> path

    val tag           : term -> term         (* annotate a term with paths *)

    val extend_thy    : theory-> theory      (* extend a theory's syntax so it
					      * can be used to print a 
					      * tagged term 
					      *)

    val string_of     : theory-> term -> string
                                             (* annotate a term and return
					      * an SGML string representatio
					      * of the tagged term *)

    val print_str     : theory-> term -> string
                                             (* annotate a term and return
					      * a string representation of 
					      * the tagged term *)

    (* the following, given a term path, `cut out' the term at that position
     * or the surrounding context *)
    val cut_term    : term -> path -> term
    val cut_context : Sign.sg-> term -> path -> cterm

  end
=


struct 


    type path= int list

    fun encode p =
    let
	fun encode [] = "0"
	  | encode (1::S) = "1"^(encode S)
	  | encode (2::S) = "2"^(encode S)
    in  "(0)"^(encode p)
    end
       
    fun decode x = 
	let fun decode ["0"] = []
	      |decode ("1"::S) = 1:: (decode S)
	      |decode ("2"::S) = 2:: (decode S)
	in decode (explode x) 
	end

    (* Bad hack -- these are the names of all variable-binding operations.
     * For these, we do not put a <SEL path> around its variable argument,
     * because otherwise the print translation doesn't translate them 
     * back anymore.
     * One should of course detect that an operation is a binder and treat
     * it accordingly, rather than hardwire the names into the code (but
     * it's only a week to go to Tapsoft :-)
     *)

    fun is_mon_binder nm = Lib.List.exists (fn str=>str= nm) 
	["All", "all", "Ex", "ex", "Ex1", "ex1", 
	 "Collect", "UNION1", "INTER1", "Ball", "Bex"]
    fun is_bin_binder nm = Lib.List.exists (fn str=>str= nm) 
	                   ["split"]	    

    fun tag t =
	let fun gen path t = (Const ("_sel",dummyT) 
			      $ Free (encode path,dummyT) 
			      $ t) 
	    fun tag' p (Const(s,t)) = gen p (Const(s,t))
	      | tag' p (Free (s,t)) = gen p (Free (s,t)) 
	      | tag' p (Var  (s,t)) = gen p (Var  (s,t))
	      | tag' p (Bound   n)  = gen p (Bound   n)
	      | tag' p (Abs(s,t,tt))= gen p (Abs(s, t, tag' (1::p) tt))
	      | tag' p (s $ t)      = 
	        let fun tag_app p ((s $ t) $ u) = 
		                     (tag_app (1::p) (s $ t)) $ (tag' (2::p) u)
		      | tag_app p ((s as Const(s0,t0)) $ 
				        (a as Abs(s1, t1, Abs(s2, t2, body))))=
			if (is_bin_binder s0) then 
			    s $ Abs(s1, t1, Abs(s2, t2, tag' (1::1::2::p) body))
			else
			    s $ tag' (2::p) a				   
		      | tag_app p ((s as Const(s0,t0)) $ Abs(s1, t1, body)) =
			if (is_mon_binder s0) then 
			    s $ Abs(s1, t1, tag' (1::2::p) body)
			else
			    s $ tag' (2::p) (Abs(s1, t1, body)) 
		      | tag_app p (s $ t) = s $ (tag' (2::p) t)
	       in  gen p (tag_app p (s $ t)) 
	       end
	in tag' [] t 
	end


    fun extend_thy thy = 
	let val nuthy= 
	    Theory.add_thyname ((!(hd(stamps_of_thy thy)))^"+")
	    (Theory.add_consts  
	     [("_sel", 
	       "['a  , 'b  ] => 'b   ", 
		   Mixfix ("(2)((0)<SEL (_)(0)>(_)(0)<\\SEL>)", [], max_pri))] 
	     thy)
	in  (SGML.attach_ext_syntax nuthy [] []; nuthy)
	end

    fun string_of th t= SGML.string_of_term (extend_thy th) (tag t)

    fun print_str th t= Sign.string_of_term (sign_of (extend_thy th)) (tag t)

    exception IllegalPath of string

    fun cut_term t hp = 
        let fun cut_term (Abs(s,t,tt)) (1::path) = Abs(s, t, cut_term tt path)
              | cut_term (Abs(s,t,tt)) []        = Abs(s, t, tt)
              | cut_term (s $ t) (1::p)          = cut_term s p
              | cut_term (s $ t) (2::p)          = cut_term t p
              | cut_term t       []              = t
              | cut_term t       (x::p)          = raise (IllegalPath 
                                                          (encode (rev hp)))
        in cut_term t (rev hp) 
        end
    
    fun cut_context sg t hp = 
	let fun gen d = Library.foldl (op $) (Bound d, 
					      map Bound ((d-1) downto 0));

            fun cut (Abs(s,t,tt)) (1::path) d = Abs(s,t,cut tt path (d+1))
	      | cut (s $ t) (1::p)          d = (cut s p d) $ t
	      | cut (s $ t) (2::p)          d = s $ (cut t p d)
	      | cut t       []              d = gen d
 	      | cut t       (x::p)          d = raise (IllegalPath 
							(encode (rev hp)));


	   fun constr_type args tt= 
		let val subterm = subst_bounds(args,tt)
		    val st_type = #T(rep_cterm(cterm_of sg subterm))
		in  Library.foldl (fn (dom,Free(_, t))=> t-->dom) 
				  (st_type,args) 
		end;
		              

	   fun red_type (Abs(s,t,tt)) (1::p) args = red_type tt p 
							(Free(s,t)::args)
	      | red_type (s $ t) (1::p) args       = red_type s p args
	      | red_type (s $ t) (2::p) args       = red_type t p args
	      | red_type t       []     args       = constr_type args t
	      | red_type t       (x::p) args       = raise (IllegalPath 
						             (encode (rev hp)));


	in  cterm_of sg  (Abs("redex", red_type t (rev hp) [], 
			               cut t (rev hp) 0))
	end
    
end

(* 
Test1: try the following operations:


goal List.thy "a * (b + c) = a + b * a + c";
val bla = concl_of (topthm());


writeln(TermSel.string_of (List.thy) bla);

TermSel.cut_term bla (TermSel.decode "212120");
TermSel.cut_term bla (TermSel.decode "2122120");
TermSel.cut_term bla (TermSel.decode "220");
TermSel.cut_term bla (TermSel.decode "0");
TermSel.cut_context sign bla (TermSel.decode "2221220");


goal  List.thy "!a b c. a & aa * (b + (c::nat) ) = aa + b * aa + c";
val bla = concl_of (topthm());
val sign = sign_of (List.thy);
writeln(TermSel.string_of ( List.thy) bla);

>>>>>>>>>>>>>>>>>>>>>
<SEL 0><SEL 20><FONT SYMB>"<\FONT>a. <SEL 1220><FONT SYMB>"<\FONT>b. <SEL 121220><FONT SYMB>"<\FONT>c. <SEL 12121220><SEL 2112121220>a<\SEL> <FONT SYMB><\FONT> <SEL 212121220><SEL 21212121220><SEL 2121212121220>aa<\SEL> * (<SEL 221212121220><SEL 21221212121220>b<\SEL> + <SEL 2221212121220>c<\SEL><\SEL>)<\SEL> = <SEL 2212121220><SEL 212212121220><SEL 21212212121220>aa<\SEL> + <SEL 2212212121220><SEL 212212212121220>b<\SEL> * <SEL 22212212121220>aa<\SEL><\SEL><\SEL> + <SEL 22212121220>c<\SEL><\SEL><\SEL><\SEL><\SEL><\SEL><\SEL><\SEL>
<<<<<<<<<<<<<<<<<<<<<

TermSel.cut_term bla (TermSel.decode "2112121220");
TermSel.cut_term bla (TermSel.decode "12121220");
TermSel.cut_context sign bla (TermSel.decode "12121220");
(* crashed !!! *)

(*     fun cut_context sg t hp =  *)
val t = bla;
val sg = sign;
val hp =  (TermSel.decode "12121220");


fun gen d = Library.foldl (op $) (Bound d, map Bound ((d-1) downto 0));	                
fun cut (Abs(s,t,tt)) (1::path) d = Abs(s,t,cut tt path (d+1))
      | cut (s $ t) (1::p)          d = (cut s p d) $ t
      | cut (s $ t) (2::p)          d = s $ (cut t p d)
      | cut t       []              d = gen d
      | cut t       (x::p)          d = raise (IllegalPath (encode (rev hp)));


fun constr_type args tt= 
    let val subterm = subst_bounds(args,tt)
        val st_type = #T(rep_cterm(cterm_of sg subterm))
    in  Library.foldl (fn (dom,Free(_, t))=> t-->dom) (st_type,args) end;
		              

fun red_type (Abs(s,t,tt)) (1::p) args = red_type tt p (Free(s,t)::args)
	      | red_type (s $ t) (1::p) args       = red_type s p args
	      | red_type (s $ t) (2::p) args       = red_type t p args
	      | red_type t       []     args       = constr_type args t
	      | red_type t       (x::p) args       = raise (IllegalPath 
						             (encode (rev hp)));

in  cterm_of sg  (Abs("redex", red_type t (rev hp) [], 
			                        cut t (rev hp) 0)) end;







Test2:
(* inludes Macro-Rewriting *)
Pretty.setmargin 60;
goal Set.thy "!x y. {x,y,z,a,b,c,d,e} Un {x,y,z,a,b,c,d,e} = {x,y,z,a,b,c,d,e}";
val term = concl_of (topthm());
val sign = sign_of(Set.thy);

writeln (TermSel.string_of Set.thy term);
val ast_hook = ref([]: ast list);
(TermSel.string_of sign term) handle AST (x,y) => (ast_hook:=y;x);

>>>>>>>>>>>>>>>>>>>
<SEL 0><SEL 20>! x. <SEL 1220>! y. <SEL 121220><SEL 21121220><SEL 2121121220>{<SEL 212121121220>x<\SEL>, <SEL 2122121121220>y<\SEL>, <SEL 21222121121220>z<\SEL>, <SEL 212222121121220>a<\SEL>, <SEL 2122222121121220>b<\SEL>, <SEL 21222222121121220>c<\SEL>, <SEL 212222222121121220>d<\SEL>, <SEL 2122222222121121220>e<\SEL>}<\SEL> Un
          <SEL 221121220>{<SEL 21221121220>x<\SEL>, <SEL 212221121220>y<\SEL>, <SEL 2122221121220>z<\SEL>, <SEL 21222221121220>a<\SEL>, <SEL 212222221121220>b<\SEL>, <SEL 2122222221121220>c<\SEL>, <SEL 21222222221121220>d<\SEL>, <SEL 212222222221121220>e<\SEL>}<\SEL><\SEL> =
          <SEL 2121220>{<SEL 212121220>x<\SEL>, <SEL 2122121220>y<\SEL>, <SEL 21222121220>z<\SEL>, <SEL 212222121220>a<\SEL>, <SEL 2122222121220>b<\SEL>, <SEL 21222222121220>c<\SEL>, <SEL 212222222121220>d<\SEL>, <SEL 2122222222121220>e<\SEL>}<\SEL><\SEL><\SEL><\SEL><\SEL>
<<<<<<<<<<<<<<<<<<<<

TermSel.cut_term term (TermSel.decode "212222121220");
TermSel.cut_term term (TermSel.decode "212222221121220");
TermSel.cut_term term (TermSel.decode "2121220");

Test 3: (* displays AST-level ... *)
(* Requires universal export of Printer 
   and Syntax ( structure Syntax (* : SYNTAX *) = !!!) *)
open Printer;

Pretty.setmargin 60;
goal Set.thy "!x y. {x,y,z,a,b,c,d,e} Un {x,y,z,a,b,c,d,e} = {x,y,z,a,b,c,d,e}";
val term = concl_of (topthm());
val sign = sign_of(Set.thy);
val syn = syn_of (TermSel.extend_thy Set.thy);
val Syntax.Syntax tabs = syn;   
val {print_trtab, print_ruletab, print_ast_trtab, prtab, ...} = tabs;
val ast = term_to_ast (Syntax.lookup_trtab print_trtab) prtab term;
val ast = term_to_ast (Syntax.lookup_trtab print_trtab)prtab(TermSel.tag term);


Test 4: (* displays AST-level ... *)
goal List.thy "sqsum xs = foldr (op +) 0 (map (%x.x * x) (map sum xs))";
val term = concl_of (topthm());
val thy2 = (TermSel.extend_thy  List.thy);

writeln (TermSel.string_of List.thy term);



KNOWN BUGS:
1.) Behandlung der Quantoren.
    <SEL 0><SEL 20>! x. <SEL 1220>! y.  ...
2.) Mysterious selection fault problem ...

<SEL 0><SEL 20><SEL 2120>sqsum <SEL 22120>xs<\SEL><\SEL> = <SEL 220>foldr <SEL 211220>op +<\SEL> <SEL 21220>0<\SEL> <SEL 2220>map <SEL 212220><FONT SYMB>l<\FONT>x. <SEL 1212220><SEL 211212220>x<\SEL> * <SEL 21212220>x<\SEL><\SEL><\SEL> <SEL 22220>map <SEL 2122220>sum<\SEL> <SEL 222220>xs<\SEL><\SEL><\SEL><\SEL><\SEL><\SEL>


*)

