(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/isa_toolkit/chooser.sml,v $
 
   The Chooser -- an application allowing selection of theories and theorems
   
   $Date: 1997/08/05 15:48:02 $
   $Revision: 1.12 $
   Author: bu (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems (BISS), Universitaet Bremen
 
  ************************************************************************** *)


(* RegExpClipChooser is the fully generic chooser. 
 *
 * Just as with the filer, there are two instantiated versions available (see
 * below): ClipChooser, where the regular expression matcher is instan-
 * tiated with Ryan Stansifer's matcher from the sml_tk toolkit, and 
 * Chooser, where additionally the clipboard is instantiated with
 * a dummy clipboard.
 *)

functor RegExClipChooser 
    (structure M : sig  val rex_filter : string -> string -> bool; 
		        exception bad_regexp
			val SEL_THY 	: ((string * theory) list) ref
    			val SEL_THMS 	: ((string * thm) list) ref
    			val refresh  : TkTypes.SimpleAction
		        structure CB : CLIPBOARD_W
		        val thyToObj : (string * theory) -> CB.obj
		        val thmToObj : (string * thm) -> CB.obj
		   end
     structure conf : GENCONF):
 sig
    open BasicTypes   
    val refresh_thm_win: SimpleAction
    val refresh_thy_win: SimpleAction
    val chooser : unit -> (string * thm) list 
end 
=
struct

local 
    open TkTypes  SmlTk BasicUtil Lib.List M 
in

structure utilWin= UtilWin(conf.uw)

(* Isabelle 94-7 backwards compatibility package :-) *)
fun freeze thm = fst(freeze_thaw thm);


val FOUND	= ref ([] : ((string * theory) * 
			    ((string * thm) list Library.option * 
					(string * thm) list Library.option))
			    list);
val THY_FOUND	= ref ([] : (string * theory) list);

(* ==================== *)
(* Chooser System State *)
(* ==================== *)

val LIMIT = 100

val THM_PAT = ref ("?XXX");
val THY_NAME_PAT = ref ("");
val THM_NAME_PAT = ref ("");
val FIL_AXMS = ref ([] : (string * thm) list);
val FIL_THMS = ref ([] : (string * thm) list);

val noaction = fn () => ();    
val dragCursor = Cursor(XCursor("fleur", NONE))  (* configure *)


(* ======================== *)
(* Searching Functionality *)
(* ======================== *)


(* utility functions *)

(* The empty string should match anything no matter what rex_filter says 
 * (some matchers don't consider the empty string a regular expression,
 *  which is inconveniet) *)

fun safe_filter "" str = true
  | safe_filter fl str = rex_filter fl str



fun select_thm n theo linesRead =
    let fun delta None      = 0
    	  | delta (Some list) = (2 + (length list)) + 1 
    	  	(* Ueberschrift plus Laenge plus Leerzeile *)
    	val (theory,(axioms,thms)) = nth(!FOUND,theo);
    	(* val Some ax    = axioms; *)
    	val deltaAxiom = delta axioms;
    	val deltaThm   = delta thms;
    	val theoL      = 2 + deltaAxiom + deltaThm;
    	val offset = n-linesRead
    in  if n>linesRead + theoL then select_thm n (theo+1) (linesRead+theoL)
    	else let
    		 val t      = if offset < 2+deltaAxiom then
    		 		(* nicht Ueberschr. und Leerzeile *)
    		 		if offset>3 andalso offset<deltaAxiom+1 then 
    		 		  let val Some ax    = axioms
    		 	          in [nth(ax,offset-2-1-1)] end (* Ueb, Leer, 0=1 *)
    		 	        else []
    		     	      else			      (* Theorem geklickt *)
    		     	      	if offset > deltaAxiom+2+1 andalso
    		     	      	   offset<deltaAxiom+1+deltaThm then
    		     	          let val Some t     = thms
    		     	          in [nth(t,offset -2-deltaAxiom-2)] end
    		     	        else []
    		     	        	(* 2:Ueb. Theorie, 2:Ueb. Theorem *)
    	     in (theory,t) end
    end;
    



(* ======================== *)
(* Theorem Selection Window *)
(* ======================== *)



val build_thm_liste = 
	let fun sel_thm_prep _= let val Mark(n,_) = readCursor "thmliste"
                                in  case select_thm n 0 0 of
		                 (_,[t]) =>( (*SEL_THMS := t::(!SEL_THMS);*)
					     (* mono-betrieb z.Zt, solange
						    keine Object-Gruppen  *)
					     SEL_THMS := [t];
				             addConf "thmliste" [dragCursor];
		    		             refresh();
		            	             writeln "theorem selected.\n")
		  		 | _  => (SEL_THMS := [])
        		        end;
	    fun drag_Thm ev = ((CB.put(M.thmToObj(hd(!SEL_THMS ))) ev (K()))
	                        handle Lib.List.Empty => ();
			        addConf "thmliste" [Cursor(NoCursor)];
			        refresh() );
	in      Listbox("thmliste",RightScb,
			[Side Left,Side Top, Fill Both, Expand true],
			[Relief Raised ,Height 20], 
			[(* Bind("<Button-3>",show_thm), *)
			 Bind("<Button-1>",sel_thm_prep),
			 Bind("<ButtonRelease-1>", drag_Thm)])
	end;



val quit_thm_liste = 
    let val mrs = fn () => closeWindow "thmsel" 
    in Button(newWidgetId(),[Fill X,Side Bottom],
	       [Text "Ok", Command mrs],[]) 
    end;


fun match(t1,s,t2) = 
    let fun freezeV(Const(a,T)) = Const(a, T)
          | freezeV(Free(a,T)) = Free(a,T)
          | freezeV(Var((a,i),T)) = Free(a, T)
          | freezeV(t as Bound _)  = t
          | freezeV(Abs(a,T,t)) = Abs(a, T, freezeV t)
          | freezeV(f$t) = freezeV f $ freezeV t 
        val f1 = safe_filter (!THM_NAME_PAT) s
    in f1 andalso could_unify(t1,freezeV t2) 
    end;


fun lookup_theory could (limit, t, ths) =
    let val pb = Logic.strip_assums_concl t;   (*delete assumptions*)
        fun filtr (limit, []) = []
	  | filtr (limit, (s,th0)::ths) =
	     let val th = case concl_of th0 of
				_ $ th => th
			      | _ =>(writeln("master of desaster: " ^ s ^"\n");
				     print_thm th0; concl_of th0);
	            
	     in
	    	if limit=0 then  [("...",th0)]
	    	else if could(pb,s, th)  then (s,th0) :: filtr(limit-1, ths)
	    	else filtr(limit,ths)
	     end
    in filtr(limit,ths)  end;


fun gmap f p []        = []
  | gmap f p (x::rest) =
  	let val r = f x
  	in if p r then r :: gmap f p rest else gmap f p rest
  	end;


fun fill_thm_win () =

    (let val f = safe_filter (!THY_NAME_PAT);
	
	(* 
	  thys: Alle gem. reg. Ausdr. gefundenen Theorien 
	*)
	val thys = filter (fn nm => f (fst nm)) (Symtab.dest(!loaded_thys));
	
	val thm_pat   = read (!THM_PAT)
			handle ERROR => (THM_PAT:= "?XXX";
				         utilWin.error 
					    "Error in theorem pattern." (K());
 				  	 read ("?XXX"));

	(* 
	  lu filtert Theoreme und Axiome 
	*)
	fun lu thms   = lookup_theory match (LIMIT, thm_pat,thms);
	
	(* 
	  thmfilter filtert fuer Theorie Axiome und Theoreme 
	*)	 
 	fun thmfilter thy =
 	  let val (name,thyInfo) = thy;
 	      val ThyInfo {theory,...}	= thyInfo;
 	      val Some thy		= theory;
 	      val axioms		= lu (axioms_of(thy));
 	      val thms			= lu (thms_of(thy));
 	      val a = if length(axioms) = 0 then None else (Some axioms);
 	      val t = if length(thms) = 0 then None else (Some thms)
 	  in  ((name,thy),(a,t)) end;
 	  
 	(* 
 	  mtPred testet, ob entweder keine Axiome oder keine Theoreme
 	  gefunden wurden 
 	*)
  	fun mtPred ((a,x),(None, None)) = false			   
 	  | mtPred ((a,x),(Some b, _)) = true
 	  | mtPred ((a,x),(_, Some b)) = true
 	  
 	(* 
 	  found: Alle gefundenen Paare [(theory,(axioms,thms))] 
 	*) 
 	val found = gmap thmfilter mtPred thys;

	(* nun SEL_THY setzen *)
	
	val sel = map fst found;

	(* 
	  generiert Axiom- oder Theoremeintraege 
	*) 	
 	fun printThmsAxioms [] = ()
 	  | printThmsAxioms (x::rest) = 
 	    (insertTextEnd "thmliste" (fst(x)^"\n");
 	     printThmsAxioms (rest))

 	(* 
 	  generiert Ueberschriften und haengt Eintraege dran 
 	*)
 	fun genItem s None = ()
 	  | genItem s (Some l) =
  	  	(insertTextEnd "thmliste" (s ^"\n");
 		insertTextEnd "thmliste" ("--------------------"^"\n");
 		printThmsAxioms l;
 		insertTextEnd "thmliste" ("\n"));
 	
 	(* 
 	  generiert fuer jede Theorie die Listboxzeilen 
 	*)
 	fun genListEntry theory =
 	 (insertTextEnd "thmliste" ("THEORY: " ^ fst(fst(theory))^"\n");
 	  insertTextEnd "thmliste" ("===================="^"\n");
 	  genItem "AXIOMS" (fst(snd(theory)));
 	  genItem "THEOREMS" (snd(snd(theory))))
 	  
 	
    in  clearText "filterentrythm";
    	insertTextEnd "filterentrythm" (!THM_NAME_PAT);
    	clearText "filterentrythy2";
    	insertTextEnd "filterentrythy2" (!THY_NAME_PAT);
	clearText "thmpattern";
	if (!THM_PAT) = "?XXX" then () 
	else insertTextEnd "thmpattern"(!THM_PAT);
    	FOUND := found;
    	clearText "thmliste";
    	app genListEntry found
    end)
handle bad_regexp=>
	(clearText "filterentrythm";
	THM_NAME_PAT:= "";
	clearText "filterentrythy2";
	THY_NAME_PAT:= "";
	utilWin.error "Bad regular expression" (K()));



val filter_label_thm = Label(newWidgetId(),[Fill X],[Text "Theorem/Axiom Filter"],[]);

val filter_entry_thm = 
    let fun cmd (_) = (THM_NAME_PAT:= readTextAll "filterentrythm";
    		      THY_NAME_PAT:= readTextAll "filterentrythy2";
		      fill_thm_win ())
    in Entry("filterentrythm",[Fill X],[],[Bind("<Return>",cmd)]) end;
    
    
val filter_label_thy = Label(newWidgetId(),[Fill X],[Text "Theory Filter"],[]);
val filter_entry_thy2 = 
    let fun cmd (_) = (THY_NAME_PAT:= readTextAll "filterentrythy2";
    		      THM_NAME_PAT:= readTextAll "filterentrythm"; 
		      fill_thm_win ());
	
    in Entry("filterentrythy2",[Fill X],[],[Bind("<Return>",cmd)]) end;


val pattern_label = Label(newWidgetId(),[Fill X],[Text "Formula-Scheme"],[]);
val pattern_entry = 
    let fun cmd (_) = (THM_PAT:= readTextAll "thmpattern";
		      fill_thm_win ())
    in  Entry("thmpattern",[Fill X],[],[Bind("<Return>",cmd)]) end;

val choose_thm_win  = (* thm_sel_win *)
     ("thmsel", [WinTitle "Choose Theorems"],
	[filter_label_thy,filter_entry_thy2,
	 filter_label_thm,filter_entry_thm,
	 pattern_label,pattern_entry,
	 build_thm_liste,quit_thm_liste],noaction);


val refresh_thm_win = fn () =>
	(if occursWin "thmsel" then () else 
	     openWindow choose_thm_win;
	 SEL_THMS := [];
	 fill_thm_win());


(* ====================== *)
(* Theorem Chooser Window *)
(* ====================== *)

val build_thy_list = 
	let fun get_sel_thy (_) = let val Mark(n,_) = readCursor "thyliste";
				      val (thyName,theory) = nth(!THY_FOUND,n)
	
   				  in  THY_NAME_PAT:=thyName;
    				      SEL_THY:=[(thyName, theory)];
				      refresh_thm_win();
				      refresh()
				  end;

	    fun drag_sel_prep ev= let val Mark(n,_) = readCursor "thyliste";
        			      val (thyName,theory) = nth(!THY_FOUND,n)
	
    			          in  THY_NAME_PAT:=thyName;
    				      SEL_THY:=[(thyName, theory)];
				      addConf "thyliste" [dragCursor]
   			          end;
	    fun drag_sel_thy ev = (CB.put(M.thyToObj(hd (!SEL_THY))) ev (K());
				   addConf "thyliste" [Cursor(NoCursor)];
				   refresh() );
	in  Listbox("thyliste",RightScb,
			[Side Left,Side Top,Fill Both, Expand true],
			[Relief Raised,Height 23],		
			[Bind("<Double-Button-1>", get_sel_thy),
			 Bind("<Button-1>", drag_sel_prep), 
			 Bind("<ButtonRelease-1>", drag_sel_thy)]) end;
	                 (* these bindings are not very good. It should select
			  * a thy with one click, and display/open it with
			  * a double click, so get_sel_thy above needs to
			  * be split in two functions.
			  *)

fun quit_but win = 
    Button(newWidgetId(),[Fill X,Side Bottom],
			[Text "Quit", Command (fn () => closeWindow win)],[]);

fun fill_thy_win () = 
    let fun getThyInfo theo = let val (name,thyInfo) = theo;
				  val ThyInfo {theory,...} = thyInfo;
 	      			  val Some thy  = theory
 	      		      in (name,thy)
 	      		      end;
 	      			 
 	fun genLine (name,thy) = insertTextEnd "thyliste" name;

        fun f x = safe_filter (!THY_NAME_PAT) (fst x)
 
    in (let val found = map getThyInfo (filter f (Symtab.dest(!loaded_thys)))
        in  clearText "filterentrythy";
	    insertTextEnd "filterentrythy" (!THY_NAME_PAT);
             clearText "thyliste";
    	    THY_FOUND := found;
 	    app genLine found
        end)
       handle bad_regexp =>(THY_NAME_PAT:= "";
			    clearText "filterentrythy";
		            utilWin.error "Bad regular expression" (K()))
    end;

val filter_label_thy = 
    Label(newWidgetId(),[Fill X],[Text "Filter"],[]);

val filter_entry_thy = 
    let fun cmd (_) = (THY_NAME_PAT:= readTextAll "filterentrythy";
		      fill_thy_win ())
    in Entry("filterentrythy",[Fill X],[],[Bind("<Return>",cmd)]) end;


val choose_thy_win = 
    let val winId = "choosersystem"
    in  (winId, [WinTitle "Choose Theory"],
	 [filter_label_thy, filter_entry_thy,
		build_thy_list,quit_but winId],noaction) 
    end;

val refresh_thy_win = fn () =>
    (if occursWin "choosersystem"
        then () 
	else openWindow choose_thy_win;
     THY_NAME_PAT := "";
     THM_NAME_PAT := "";
     fill_thy_win());

val initwin = 
    let val winId = "main"
    in  (winId, [WinTitle "Choose Theory"],
	[filter_label_thy, filter_entry_thy,
		build_thy_list,quit_but winId],fill_thy_win) 
    end;


val chooser  = fn () => (SEL_THMS:=[]; SEL_THY:=[]; THY_NAME_PAT:="";
			 startTcl [initwin];
			 !SEL_THMS);


end
end



functor ClipChooser 
    (structure M : sig
			val SEL_THY  : ((string * theory) list) ref
    			val SEL_THMS : ((string * thm) list) ref
    			val refresh  : TkTypes.SimpleAction
		        structure CB : CLIPBOARD_W
		        val thyToObj : (string * theory) -> CB.obj
		        val thmToObj : (string * thm) -> CB.obj
		   end
     structure conf : GENCONF):
	 sig
	     val refresh_thm_win: TkTypes.SimpleAction
	     val refresh_thy_win: TkTypes.SimpleAction
	     val chooser : unit -> (string * thm) list 
	 end  = 
     RegExClipChooser
	 (structure M= 
	      struct
 		  exception bad_regexp 
		  fun rex_filter s t = (Rex.rex s t
				        handle Rex.bad_rex => raise bad_regexp);
		  val SEL_THY        = M.SEL_THY
		  val SEL_THMS       = M.SEL_THMS
		  val refresh        = M.refresh
		  structure CB       = M.CB
		  val thyToObj       = M.thyToObj
		  val thmToObj       = M.thmToObj
	      end
	  structure conf= conf)


functor Chooser 
    (structure M : sig 	                
			val SEL_THY 	: ((string * theory) list) ref
    			val SEL_THMS 	: ((string * thm) list) ref
    			val refresh : TkTypes.SimpleAction
		   end
     structure conf :GENCONF ) : 
	 sig
	     val refresh_thm_win: TkTypes.SimpleAction
	     val refresh_thy_win: TkTypes.SimpleAction
	     val chooser : unit -> (string * thm) list 
	 end  = 
    ClipChooser
        (structure M= 
	     struct
		 val SEL_THY  = M.SEL_THY
		 val SEL_THMS = M.SEL_THMS
		 val refresh  = M.refresh
		 structure CB = DummyCB
		 val thyToObj = BasicUtil.K()
		 val thmToObj = BasicUtil.K()
	     end
	 structure conf= conf)
				    
	 
	 

