(* ***************************************************************************

   $Source: /home/uniform/rep/isa_gui/isa_win/appl.sml,v $


   IsaWin -- the IsaWin application for GenGUI
   
   $Date: 1998/09/18 15:19:18 $
   $Revision: 1.23.4.2 $
   Author: cxl (Last modification by $Author: kol $)

   (C) 1996, Bremen Institute for Safe Systems (BISS), Universitaet Bremen

  ************************************************************************** *)
 

functor IsaWinAppl(conf: ISAWINCONF) :
    sig include APPL_SIG
	val startProof  : Thy.theory* TkTypes.Coord* string* thm list-> string-> 
	                                                         new_object list
	val startCProof : cterm* string-> object
	    
	val prot : unit-> string 

    end =

struct

   
   open TkTypes SmlTk IsaWinTypes Lib.List

   structure utilWin  = UtilWin(conf.uw)
   structure enterWin = EnterWindows(conf.uw)
   structure conArea  =  ConArea(conf)

   open conArea

   type object     = history.state object'
   type objectlist = object list

   type new_object = history.state new_object'

   fun prot () = 
       BasicUtil.concatWith "\n" (map #2 (history.past()))

   fun freeze thm = fst(freeze_thaw thm)


(* --- Icons! ------------------------------------------ *)

   fun icon ob = 
       let fun iconName (thm_type)           = "theorem.gif"
	     | iconName (thy_type)           = "theory.gif"
	     | iconName (simpset_type)       = "simpset.gif"
	     | iconName (claset_type)        = "claset.gif"
	     | iconName (proof_type)         = "proof.gif"
	     | iconName (tac_type)           = "tac.gif"
	     | iconName (subst_type)         = "sub.gif"
	     | iconName (text_type) 	     = "text.gif"     
       in Icons.getIcon(conf.iconPath(), iconName (obj_type ob))
       end


(* --- Operations -------------------------------------- *)


(* -- 1. text conversions and show -- *)

fun proofText ((past, future), prf) = 
    (*
     * In fact, this function belongs to isa_toolkit/focus.sml. 
     * Or then again, maybe not. 
     *)
    let	val _ = (Thy.push_proof(); Thy.restore_proof prf)
    	val m = nprems_of (topthm())
    	val g = Sign.string_of_term (sign_of
                               	(theory_of_thm(topthm())))
                               	(concl_of(topthm()))
        fun subgoal i = 
	    let val t = Sign.string_of_term (sign_of
		                            (theory_of_thm(topthm()))) 
		                                           (getgoal i)
	    in " "^(makestring i)^". "^ t^"\n\n"
	    end
        val hs = "\n\nPast "^(makestring (length past))^" steps, future "^
	          (makestring (length future))^" steps.\n";
    in (Thy.pop_proof();
	if m = 0 then g^"\n\nNo subgoals!"
	else g^"\n"^ (BasicUtil.concat (map subgoal (BasicUtil.--(1,m)))) ^ hs
       )
    end


fun kindText introThm  = "an introduction rule"
  | kindText elimThm   = "an elimination rule"
  | kindText destrThm  = "a destruction rule"
  | kindText rewrRule  = "a rewriting rule or definition"
  | kindText unknown   = "not known or set" 

       
(* this sort of display-text widget used belo should go into utilwin *)
fun showThm(nm, th, k) = 
    let val txt= "<FONT BOLD>Name : "^nm^"<\\FONT>"^
	         "\n\nKind : "^(kindText k)^
		 "\n\n"^(SGML.string_of_term (theory_of_thm th) 
			 (#prop (rep_thm (freeze th))))
	val wid = newWidgetId()
	val win = newWinId()
	val (wtxt, ann)= StdMarkup.getAnnotation wid txt
	val quitBut =  Frame(newWidgetId(),
			     [Button(newWidgetId(), [Side Right],
				     [Text "Dismiss", 
				      Command (fn () => closeWindow win),
				      Font conf.uw.buttonFont], [])
			     ], [Side Bottom, Fill X], [], [])
	val textWid =  TextWid(newWidgetId(), NoneScb, wtxt, ann,
			      [Side Top, Fill Both], 
			      [Relief Flat, Borderwidth 10], [])
    in  openWindow(win, [WinTitle nm], [quitBut, textWid], fn()=> ())
    end


(* -- special operations -- *)

(*
 * start a proof in theory thy given text txt and definitions defs. 
 * 
 * returns a list of a proof object to be placed "South" of parameter `where',
 * and the metahypotheses of the proof as a list of theorem objects to be 
 * placed East of `where', if everything goes well.
 *)
fun startProof (thy, where, name, defs) txt = 
    let val _    = Thy.push_proof()
	val prems= Thy.goalw thy defs txt
	val prfob= (isaPrf(history.save(), ref name), where, South)
	val premobs= map (fn (thm, i)=> 
			  (isaThm(thm, ref unknown, 
				  ref ("*Hyp "^(Int.toString i)^"*")), 
			   where, East))
	                    (Lib.ListPair.zip (prems, (1 upto (length prems))))
     in  (pop_proof(); prfob::premobs) 	 
     end handle ERROR =>
	     (utilWin.error "Isabelle can't parse the proof text correctly." 
	       (K()); []) 

(* same as before, but query for text first *)
fun startProofInThy (isaThy(theta, nm), where) objcont =
    utilWin.enterText "Text Edit" "Please enter statement to be proven." "" 
                      30 5 ((Lib.List.app objcont) o 
			        (startProof (theta, where, "*new proof*", [])))
			                          (K())

(* same as before, but goal is given as a cterm, and no coordinates are
 * specified-- this is for use with TAS and the clipboard *)
fun startCProof (ct, name) = 
    (Thy.push_proof();
     goalw_cterm [] ct handle ERROR =>
           (utilWin.error "Isabelle can't initiate the proof." 
	                  (fn _=> ignore(Thy.pop_proof())); 
	    []);	   
     let val prfob= isaPrf(history.save(), ref name)
     in  (Thy.pop_proof(); prfob)
     end
    )


fun editTxt ob = utilWin.enterText (sel_name ob) 
                     "Edit text.\nClick OK to commit edit, or cancel to abort."
		     (selText ob) 80 25 (setText ob) (K())

(*
 * needs to be extend for lists:
 * th RS th1...thn = th RS1 th1 RS2 th2 ... RSn thn ?
 *)

fun RSkind introThm introThm  = introThm
  | RSkind elimThm  elimThm   = elimThm
  | RSkind destrThm destrThm  = destrThm
  | RSkind rewrRule rewrRule  = rewrRule
  | RSkind _        _         = unknown

fun oneRS (isaThm(thm1, k1, n1), where) ((isaThm(thm2, k2, n2)):: _ ) =
  let val nuthm = thm2 RS thm1
      val nunm  = (!n2)^" RS "^(!n1)
  in  [(isaThm(nuthm, ref (RSkind (!k1) (!k2)), ref nunm), where, South)]
  end handle THM _ => (utilWin.error "Could not resolve:" (K()); [])



fun show (isaPrf(prf, nm))   = utilWin.display (!nm) 200 (SOME (Normalfont[]))
                                               (proofText prf) (K())    
  | show (isaThm(th, k, nm)) = showThm(!nm, th, !k)
  | show (isaCS(cs, sf, nm)) = ((* print_cs cs *))
  | show (isaSimpS(ss, nm))  = ((* map thmText rep_ss ss ... *))
  | show (isaThy(theta, nm)) = ((* compile, fire up netscape and display :-) *))  
  | show (isaTac(tic, nm))   = utilWin.warning "Can't display tactics yet."(K()) 
  | show (substObj(s, nm))   = enterWin.subst("Substitution "^ !nm, !s, 
					      fn nusubst => s:= nusubst)
  | show (textObj(t, nm))    = utilWin.display (!nm) 400 NONE (!t) (K())



fun stat   _ = ()

fun delete _ = ()

fun mon_ops text_type = [(fn (ob, wh) => fn f => editTxt ob, "Edit text.")]
  | mon_ops thy_type  = [(startProofInThy, "Start a proof in this theory")] 
  | mon_ops claset_type = 
    let fun setSafe b (isaCS(_, safe, _), _) _ = safe := b
    in  [(setSafe true, "Adding rules is safe"),
	 (setSafe false, "Adding is is unsafe")]
    end
  | mon_ops thm_type  = 
    let fun setKindThm t (ob, _) _ = setThmKind ob t
    in  [(setKindThm introThm, "Introduction thm"),
	 (setKindThm elimThm,  "Elimination thm"),
	 (setKindThm destrThm, "Destruction thm"),
	 (setKindThm rewrRule, "Rewrite rule"),
	 (setKindThm unknown,  "Unknown/undefined")]
    end
  | mon_ops _            = []


fun bin_ops (thm_type, thm_type)  = SOME oneRS
  | bin_ops (thy_type, text_type) =            
            let fun startPrf (isaThy(thy, _), wh) (textObj(t,  n) :: _ ) = 
                             startProof(thy, wh, !n, []) (!t)
	    in SOME startPrf 
	    end

  | bin_ops (thm_type, subst_type) =
            (* instantiate a theorem with a substitution *)
            let fun instTheorem(isaThm(thm, kind, nm), where) sobjs = 
		let val substs = Lib.List.concat(map selSubst sobjs)
		    val sg     = #sign(rep_thm(topthm())) 
			         (* This is debatable -- but the only
				  * other alternative I can think of is
				  * to use thm's signature which is an even
				  * worse alternative. 
				  *)
		in  [(isaThm(read_instantiate_sg sg substs thm, kind, 
			     ref ((!nm)^"*")), where, South)]
		     handle (THM(e, _, _)) => 
			  (utilWin.error ("Instantiation failed:\n\n"^e) (K());
			   [])
			  | ERROR =>
			  (utilWin.error ("Instantiation failed.\n") (K());
			   [])	
		end
	    in SOME instTheorem
	    end

  | bin_ops (simpset_type, thm_type)  = 
            (* add a list of theorems to a simplifier set *)
            let fun augmentSimpSet (isaSimpS(sset, nm), wh) ithms =
		let val thms= map selThm ithms
		    val nms = map sel_name ithms
		    val nuname= (!nm)^"+"^(BasicUtil.concatWith " " nms)
		in [(isaSimpS(sset addsimps thms, ref nuname), wh, South)]
		    (* EXCEPTIONS ?! *)
		end
	    in SOME augmentSimpSet
	    end

  | bin_ops (simpset_type, simpset_type) =
            (* merge two simplifier sets *)
            let fun mergeSimpSets (isaSimpS(s1, nm1), wh) 
				  (isaSimpS(s2, nm2) ::_ ) =
		    [(isaSimpS(merge_ss(s1, s2), ref ((!nm1)^"+"^(!nm2))), wh, South)]
	    in SOME mergeSimpSets 
	    end

  | bin_ops (claset_type, simpset_type) =
           (* merge a simpset into a claset *)
           let fun mergeSimpToClaSet (isaCS(cs, safe, nm), wh)
	                             (isaSimpS(s1, n1):: _) =
		 [(isaCS(cs addss s1, safe,
			 ref ("addss("^(!nm)^", "^(!n1)^")")), wh, South)]
	   in  SOME mergeSimpToClaSet 
	   end

  | bin_ops (claset_type, theorem_type) =
           (* add a theorem to a claset. Depending on the ``state'' of the
	    * claset, theorems are added as safe or unsafe. Bit of a hack,
	    * that, but I can't think of anything better at the mo. 
	    *)
            let fun addThms(isaCS(cs, safe, nm), wh) ithms =
		let val thms = map selThm ithms
		    val nms = BasicUtil.concatWith " " (map sel_name ithms)
		    val nuname= (!nm)^"+"^nms
		    fun ifsafe(x,y) = if (!safe) then x else y
		    val (addThms, descr) = 
			case (thmListKind ithms) of
			    introThm => ifsafe((op addSIs, "safe introduction rule"),
				               (op addIs, "unsafe introduction rule"))
			  | elimThm  => ifsafe((op addSEs, "safe elimination rule"),
					       (op addEs, "unsafe elimination rule"))
			  | destrThm => ifsafe((op addSDs, "safe destruction rule"), 
					       (op addDs, "unsafe destruction rule"))
			  | rewrRule => ifsafe((op addSIs, "safe introduction rule"),
				               (op addIs, "unsafe introduction rule"))
			  | unknown  => ifsafe((op addSIs, "safe introduction rule"),
				               (op addIs, "unsafe introduction rule"))
		    val suffix  = if (length ithms) > 1 then "s" else ""
		    val infotxt = "Theorem"^suffix^" "^nms^" added to claset "
			          ^(!nm)^" as "^descr^suffix^".\n"
			          
		in (utilWin.display ("Simpset "^(!nm)) 400 
		                    (SOME (Normalfont[])) infotxt (K());
		    [(isaCS(addThms(cs, thms), safe, ref nuname), wh, South)])
		end
	    in SOME addThms
	    end

  | bin_ops (_, _) =  NONE


fun init () = [(* initial objects are placed ``externally'' *)]


(* --- other substructures: configuration, construction area, clipboard --- *)

   structure Conf= conf
  
   structure WS = conArea

   structure CB = Clipboard(struct type obj= objectlist end)

end
