
(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/isa_win/induct.sml,v $
 
   A general induction tactic for IsaWin. 
 
   $Date: 1996/12/19 22:36:58 $
   $Revision: 1.3 $
   Author: Burkhart Wolff (Last modification $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

(* *********************************************************************** *)
(*									   *)
(* Bugs: I'm not sure if insert and remove are really what you want in 	   *)
(*	 situations after a reload.				 	   *)
(*	 I'm not sure if a best-fit strategy for type-matches should	   *)
(*	 be implemented.						   *)
(*	 This implementation assumes Isabelle/HOL as basis.		   *)
(*									   *)
(* *********************************************************************** *)

(* *********************************************************************** *)
(*									   *)
(* Example:

open induct_chooser;

goal Arith.thy "(a::nat)+b = b+a";
by(INDUCT "" "a" 1) 
   handle Induct_ambiguous s => writeln (foldl (op ^)("",s)) ;
by(INDUCT "natural induction" "a" 1);

- by(INDUCT "" "a" 1) 
   handle Induct_ambiguous s => writeln (foldl (op ^)("",s)) ;
natural inductionwell-founded induction
val it = () : unit

- by(INDUCT "natural induction" "a" 1);
Level 1
a + b = b + a
 1. 0 + b = b + 0
 2. !!k::nat. k + b = b + k ==> Suc k + b = b + Suc k

Types:
  a, b :: nat
val it = () : unit


Mit etwas Aufwand waere auch sowas wie ne best-fit strategie auf Typen fuer
die Disambiguierung moeglich ...

*)
(*									   *)
(* *********************************************************************** *)


structure InductChooser :
  sig
    exception Induct of string
    exception InductAmbiguous of string list
		(* for embedding in interactive environment *)
    val INDUCT : string -> string -> int -> tactic
    val insert    : string * thm -> unit
    val remove    : string -> unit
  end
=

struct

open Lib.List

exception Induct of string;
exception InductAmbiguous of string list;


val InductTable = ref
       [("natural induction",nat_induct),  (* nat_ind_tac), *)
	("structural induction over list", list.induct), (* list.induct_tac), *)
	("structural induction over sexp", sexp.induct)
	(* ("well-founded induction", wf_induct) *) ];


fun  get_induct_head (Const ("Trueprop",_) $ (Var(_,_) $ Var (s,t))) = (s,t)
   | get_induct_head _ = raise (Induct "not induct head");


fun  mk_induct_tac fparm ind_thm = 
	(fn act_parm => fn n => res_inst_tac [(fparm,act_parm)] ind_thm n);

fun  grab_type_of(v, Free(n,t))    = if v = n then (Some t) else None
   | grab_type_of(v,Abs(_,_,t))    = grab_type_of(v,t)
   | grab_type_of(v,f$t)           = (case grab_type_of(v,f) of
                                         None => grab_type_of(v,t)
			               | s    => s)
   | grab_type_of(v, _)            = None;


fun is_IndParmType_eq_Type thym (ty,ind_thm) =
    let val ((m,_),ty2) = get_induct_head (concl_of ind_thm);
        val {tsig,...} = Sign.rep_sg(sign_of thym);
        fun typ_unify (ty1, ty2) = (Type.unify tsig 0 [] (ty1, ty2); true)
                                   handle TUNIFY => false;
				   (* 0 ?!? *)
    in  subthy((theory_of_thm ind_thm),thym) andalso
        typ_unify (ty,ty2)
    end;



fun INDUCT induct_filter ind_par n = 
    let val thym = theory_of_thm(topthm()) 
        val prem = (nth(prems_of(topthm()),n - 1)) 
		    handle Nth => raise (Induct "No legal subgoal")
        val ty   = (case grab_type_of(ind_par,prem) of
                      Some t => t
		    | Nil    => raise (Induct "Illegal induction parameter"))
	val i_fi = if induct_filter = "" then (fn _ => true) 
                   else (fn x => (fst x) = induct_filter)
        fun g x  = is_IndParmType_eq_Type thym (ty,snd x) andalso i_fi x
        val inds = filter g (!InductTable);
    in  case  inds of
          [] => raise (Induct "No induction scheme found") 
       |  [(_,ind_thm)] =>((mk_induct_tac 
			    ((fst o fst o get_induct_head o concl_of) ind_thm)
			    ind_thm) 
			  ind_par n)
       | _ => raise (InductAmbiguous (map fst inds))
    end




fun remove ind_name = 
    (InductTable:= filter (fn (n,_) => n = ind_name) (!InductTable))


fun insert (ind_name:string,ind_thm) = 
(* find and overwrite, if thy and ind_name identical, else insertion *)
    let fun g(m,th)    = m = ind_name andalso 
		         eq_thy(theory_of_thm ind_thm, theory_of_thm th) 
        val nstore     = filter g (!InductTable)
    in  InductTable := (ind_name,ind_thm) :: nstore end

end
