
(* ***************************************************************************

   $Source: /home/uniform/rep/isa_gui/isa_win/isawin.sml,v $

   IsaWin -- the main module. 
   
   $Date: 1998/09/11 09:08:46 $
   $Revision: 1.18.4.1 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems (BISS), Universitaet Bremen
 
  ************************************************************************** *)

(* The interface to TAS (or any other application needing to 
 * prove things). This interface works via IsaWin's clipboard. The other 
 * application can `put' goals (in form of cterms) on it, which appear as
 * fresh proof objects in IsaWin. In the other direction, the application can
 * `get' theorems (which may be produced by the finisishing the proof or
 * which may just be other theorems). 
 *)

signature ISA_CLIPBOARD =
    sig
	exception Empty

	val putGoal    : cterm* string-> TkTypes.TkEvent-> unit
	val getTheorem : TkTypes.TkEvent -> thm
    end


signature ISAWINSIG= 
    sig
	val prot      : unit-> string

	val mainWin  : TkTypes.WinId* TkTypes.WinConfigure list-> 
	                                                        TkTypes.Window
	val go : unit-> unit

	structure CB : ISA_CLIPBOARD
    end

signature ISAWININST=
    sig
	val initConf : { thms    : (thm * IsaWinTypes.thmKind * string) list,
			 simpsets: (simpset* string) list,
			 clasets : (claset* string) list,
			 theories: (Thy.theory* string) list,
			 goals   : (Thy.theory* thm list* string* string) list,
			 substs  : ((string* string) list* string) list,
			 texts   : (string* string) list,
			 eta_contract : bool
		       }
	include ISAWINCONF
    end


functor IsaWin(conf: ISAWININST) : ISAWINSIG = 
struct

    open TkTypes IsaWinTypes BasicUtil SmlTk

(* instantiate Generic User Interface *)

    structure IsaWinAppl= IsaWinAppl(conf)

    structure IsaGUI  = GenGUI(structure appl= IsaWinAppl)

    structure utilWin= UtilWin(conf.uw)
    structure enterWin= EnterWindows(conf.uw)

(* Cooperating Tools *)

   structure Filer = 
       ClipFiler(structure M= 
	  struct 
	     structure CB= IsaWinAppl.CB
	     fun fileToObj (dir, file) =
		 let val filenm= dir^"/"^file
		     val objnm = ref ("File: "^file)
		     val txt  = let fun read_file si = 
			                   if end_of_stream si then ""
					   else (input_line si)^"\n"^(read_file si)
				    val is  = open_in filenm
				    val txt = read_file is
				    val _   = close_in is
				 in txt
				 end handle Io error => "Error: "^(#name error)
		 in [textObj(ref txt, objnm)]
		 end	     
	     fun dirToObj dir =
		 [textObj(ref dir, ref "Directory")]
	  end)


(* dito chooser *)

   val SEL_THY 	 = ref ([]:(string * theory)list)
   val SEL_THMS  = ref ([]:(string * thm) list)

   structure Chooser = 
       ClipChooser(structure M= 
	  struct 
	     val SEL_THY = SEL_THY
	     val SEL_THMS= SEL_THMS
	     structure CB= IsaWinAppl.CB
	     fun thyToObj (nam, thy) = [isaThy(Thy.thyToThy thy, ref nam)]
	     fun thmToObj (nam, thm) = [isaThm(thm, ref unknown, ref nam)]
	     fun refresh _ = ()
	  end
         structure conf= conf)

(* --- the IsaClipboard ------------------------------------------------- *)

   structure CB = 
       struct
	   open IsaWinAppl.CB
	   
	   fun putGoal(ct, name) ev = 
	       put ([IsaWinAppl.startCProof (ct, name)]) ev (K())  
	    
	   fun getTheorem ev = 
	       case (copy ev) of
		   isaThm(th, _, _)::_ => th
		 | _ => raise Empty

       end


(* ---- the main window & menue ----------------------------------------- *)


   fun cmdQuit win () = 
       utilWin.confirm "Do you really want to quit?" 
                        (BasicUtil.P closeWindow win)

    fun cmdNew ()  = 
	let  fun cmd str = IsaGUI.intro(textObj(ref str, ref "my goal"), 
					   (100, 50), South)
	in   utilWin.enterText "Edit New Proof" "Enter goal to be proven:" ""
			       40 5 cmd (K())
	end

    fun loadCont (a,b) = (let fun prefixThy str = 
                                 let val in_str = rev (explode str)
                                     val out_str = drop (4, in_str);
                                 in  implode (rev(out_str)) end;
                          fun more str  = 
                                    let val in_str = open_in(str);
                                    in  while not(end_of_stream in_str) do
                                          insertTextEnd "editwid" 
                                          (inputc in_str 100)
                            end;
 
    in
    (clearText "editwid";
     if (a^b) = ""
     then utilWin.error "File is empty" (K())
     else more (a^"/"^b) ;use_thy (a^"/"^ prefixThy(b)))
     handle ERROR => utilWin.error "Could not find file .thy" (K())
    end); 
 

    fun cmdLoad () =  Filer.enter_file loadCont

    fun cmdNewSubst () =
	let fun placeNewSubst s = IsaGUI.intro(substObj(ref s, ref "*new*"),
					       (100, 50), East)
	in  enterWin.newSubst("Edit new substitution.", [""], placeNewSubst)
	end

    fun cmdNotImpl() = utilWin.error "Functionality not yet implemented.\n" 
	                             (K())

    val menuWidth = 8

    fun fileMenue win = 
	Menubutton(newWidgetId(), false, 
 	       [MCommand[Text "New",  Command cmdNew],
		MCommand[Text "Load", Command cmdLoad],
		MCommand[Text "Save", Command cmdNotImpl],
		MSeparator,
		MCommand[Text "Edit", Command cmdNotImpl],
		MSeparator,
		MCommand[Text "Quit", Command (cmdQuit win)]
	       ], [Side Left], [Text "File", Width menuWidth],[]);
	

    val searchMenue = 
	Menubutton(newWidgetId(), false, 
		 [MCommand[Text "Theories", Command Chooser.refresh_thy_win],
		  MCommand[Text "Theorems", Command Chooser.refresh_thm_win],
		  MCommand[Text "Simplifier sets", Command cmdNotImpl],
		  MCommand[Text "ClassSets", Command cmdNotImpl]
		 ], [Side Left], [Text "Search", Width menuWidth],[]);


    val editMenue = 
	Menubutton(newWidgetId(), false, 
		   [MCommand[Text "Cut", Command cmdNotImpl],
		    MCommand[Text "Copy", Command cmdNotImpl],
		    MCommand[Text "Paste", Command cmdNotImpl],
		    MCommand[Text "Find", Command cmdNotImpl],
		    MSeparator,
		    MCommand[Text "New Substitution", Command cmdNewSubst]
		   ], [Side Left], [Text "Edit", Width menuWidth],[]);


    (* --- setting of Isabelle options via the options menu ----- *)

    fun toggleVar(tclVar, isaVar)= 
	isaVar:= (readVarValue tclVar)= "1"
	
    val varTypes = "types2"
    val varBrackets = "brackets2"
    val varEta = "eta2"
	
    fun toggleTypes () =    (toggleVar(varTypes, show_types);
			     toggleVar(varTypes, show_sorts))
    fun toggleBrackets () = toggleVar(varBrackets, show_brackets)
    fun toggleEta() = toggleVar(varEta, eta_contract)
	
    fun initVars () = 
	(show_types:= false; show_sorts:= false; setVarValue varTypes "0";
	 show_brackets:= false; setVarValue varBrackets "0";
	 if #eta_contract(conf.initConf) then
	     (eta_contract:= true; setVarValue varEta "1")
	 else
	     (eta_contract:= false; setVarValue varEta "0")
	)
	
    val optMenue =
	Menubutton(newWidgetId(), false,
		   [MCheckbutton[Text "Full Bracketing",
				 Command toggleBrackets,
				 Variable varBrackets],
		    MCheckbutton[Text "Show Sorts& Types",
				 Command toggleTypes,
				 Variable varTypes],
		    MCheckbutton[Text "Eta-Contraction",
				 Command toggleEta,
				 Variable varEta]
		    ], [Side Left], [Text "Settings"], [])
	
	

    (* there's room for improvement in the following, too :-) *)

    fun cmdHelpIsabelle () = 
	K() (Unix.execute
	     ("/usr/local/X11/bin/netscape",
	      ["-remote",
	       "openURL(http://www.cl.cam.ac.uk/Research/HVG/isabelle.html)"]))

	
    fun cmdHelpIsaLogics () =
	K() (Unix.execute
	     ("/usr/local/X11/bin/netscape",
	      ["-remote",
	      "openURL(http://www.informatik.uni-bremen.de/~bu/isa_doc/index.html)"]))


    val helpMenue = 
	Menubutton(newWidgetId(), false, 
		   [MCommand[Text "About Isabelle", Command cmdHelpIsabelle],
		    MCommand[Text "About Isabelle's Logics", Command cmdHelpIsaLogics]
		   ], [Side Right], [Text "Help"], []);


    fun mainMenu win = 
	Frame(newWidgetId(), 
	      [fileMenue win, editMenue, searchMenue, optMenue, helpMenue],
	      [Side Top, Fill X, Expand true],
	      [Relief Raised, Borderwidth 2], [])

    fun initialObjs () = 
	let 
	    val objs= conf.initConf
	    fun putThm (t, k, nm)= (isaThm(t, ref k, ref nm),
				                conf.thmsHere, East)
	    fun putSiSe(s, nm)=    (isaSimpS(s, ref nm),
						conf.simpsHere, East)
	    fun putCS(cs, nm)=     (isaCS(cs, ref false, ref nm),
						conf.csHere, East)
	    fun putThy(thy, nm)=   (isaThy(thy, ref nm),
						conf.thysHere, East)
	    fun putSubst(subst, nm)= (substObj(ref subst, ref nm),
						  conf.substHere, East)
	    fun putText(tx, nm)=   (textObj(ref tx, ref nm),
						conf.txtHere, East)
	    fun putPrf(thy, defs, goal, nm)= 
		(IsaWinAppl.startProof(thy, conf.prfsHere, nm, defs) goal)
	in  
	    ((map putThm(#thms objs)) @
	     (map putSiSe(#simpsets objs)) @
	     (map putCS(#clasets objs)) @
	     (map putThy(#theories objs)) @
	     (Lib.List.concat (map putPrf(#goals objs))) @
	     (map putSubst(#substs objs)) @
	     (map putText(#texts objs)))
	end
	    
    fun mainWin(win, conf) = (win, conf, 
				    [mainMenu (mainWinId()),
				     IsaGUI.main_wid (mainWinId())], 
				     fn()=> (initVars();
					     IsaGUI.init 
					       (IsaGUI.initial_state());
					     Lib.List.app
					       IsaGUI.intro (initialObjs())))



    fun go () = 
	let val mw = mainWin(mainWinId(), 
			     [WinTitle "Isabelle/HOL Window System", 
			      WinGeometry (NONE, SOME conf.isaWinXY)])
	in  output(std_out, SmlTk.startTclExn([mw]))
		       handle IsaGUI.GenGUI err => 
			        raise Fail("GenGUI error: "^err)
     
	end

    (* external access to history *)
    val prot = IsaWinAppl.prot


end


