(* Project	: Externalized Mini-Spectrum 
   Author 	: B. Wolff
   This theory	: Operations for Environments *)


Fmap = TAS_HOL + 


types   ('a,'b) Fmap 


arities	Fmap :: (term,term)term

(* right - unique sets of pairs : *)
consts 	runS :: "('a*'b) set => ('a*'b) set set"

inductive "runS(A)"
  intrs
    emptyI  "{} : runS(A)"
    insertI "[| b: runS(A); fst(a) ~: fst``b |]				\
	     \  ==> 	insert a b : runS(A)"


consts	Rep_Fmap :: "('a,'b)Fmap => ('a*'b)set"
	Abs_Fmap :: "('a*'b)set  => ('a,'b)Fmap"

rules	Rep_Fmap 	 "Rep_Fmap(m) : runS(X)"
  	Rep_Fmap_inverse "Abs_Fmap(Rep_Fmap(m)) = m"
  	Abs_Fmap_inverse "m : runS(A) ==> Rep_Fmap(Abs_Fmap(m)) = m"


consts	Map	 :: "'a set * 'b set => ('a,'b) Fmap set"
	"@fmzero":: "('a,'b)Fmap"			      ("{|}")
	fmmk     :: "['a,'b] => ('a,'b)Fmap"	 	      
	"^"      :: "[('a,'b)Fmap, 'a] => 'b"		      (infixl 80) 
	dom	 :: "('a,'b)Fmap => ('a)set"
	codom	 :: "('a,'b)Fmap => ('b)set"
	"<+"	 :: "[('a,'b)Fmap,('a,'b)Fmap]=>('a,'b)Fmap"  (infixl 65)
	map	 :: "['b => 'c, ('a,'b)Fmap] => ('a,'c)Fmap"

(* Installing syntax for {|x:=X; ...; z:= Z|} 				*)
types   ('a,'b) Fmaps

consts	""	 :: "('a,'b)Fmap => ('a,'b)Fmaps"	      ("_")    
	"@Fmmk"  :: "['a,'b] => ('a,'b)Fmap"	 	      ("_ := _")
	"@Enum"  :: "[('a,'b)Fmap,('a,'b)Fmaps]=>('a,'b)Fmaps"("_;/ _")
	"@Finmap":: "('a,'b)Fmaps => ('a,'b)Fmap"             ("{|(_)|}")

translations
	"{|x := X; xs|}"   == "(fmmk x X) <+ {|xs|}"
	"{|x := X|}"       == "(fmmk x X)"
	"{|x := X; xs|}"   <= "{|x := X|} <+ {|xs|}"

(* End Installation			 				*)


defs	Map_def		 "Map      == (%(U,V).{M.dom(M) <= U & 	            \
			 \		         (ALL b. b:dom(M)-->M^b:V)})"
	fmzero_def 	 "{|} 	   == Abs_Fmap({})"
	fmmk_def	 "({|x:=X|})== (Abs_Fmap { (x, X) }) "
	lkup_def	 "A ^ x    == @y. (x,y) : Rep_Fmap(A)"
	dom_def		 "dom(A)   == (fst``(Rep_Fmap(A)))"
	codom_def	 "codom(A) == (snd``(Rep_Fmap(A)))"
	ovwr_def	 "A <+ B   == @C. (dom(C) = dom(A) Un dom(B)) &	\
			 \		  (!x. x:dom(C) --> 		\
			 \		          C^x =(if x:dom(A)     \
			 \				then A^x else B^x))"
	map_def		 "map f A == @C. (dom(C) = dom(A)) &		\
			 \		  (!x. x:dom(A) --> C^x = f(A^x))"

(*  The definition fmmk_def leaves the lookup underspecified for *{|} *)


rules

(*  This should be theorems to be proved when I have more time. *)

	lkup_ovwr1	"({|x:=X|} <+ A) ^ x = X"
	lkup_ovwr2	"x~=y ==> ({|x:=X|} <+ A) ^ y = A ^ y"
    
	ovwr_ln		"{|} <+ A = A"
	ovwr_rn		"A <+ {|} = A"
	ovwr_c 		"(dom(A) Int dom(B) = {}) ==> A <+ B = B <+ A"
	ovwr_a 		"A <+ (B <+ C) = A <+ B <+ C"
	ovwr_i 		"A <+ A = A"

	upd_c  		"x~= y ==>{|x:=X;y:=Y|} <+A={|y:=Y;x:=X|} <+A"
	upd_i  		"{|x:=X; x:=Y|} <+ A = {|x:=X|} <+ A"
	dom_mt		"dom({|}) = {}"
	dom_upd		"dom({|x:=X|}) = {x}"
	dom_ovwr	"dom(A <+ B) = dom(A) Un dom(B)"
	codom_mt	"dom({|}) = {}"
	codom_upd 	"codom({|x:=X|}) = {X}"
	codom_ovwr	"codom(A <+ B) = codom(A) Un codom(B)"


	fmap_induct	"[| P({|}); !A x y. P(A) --> P({|x:=y|} <+ A)|] ==> P(A)"
	
	map_1		"map f {|} = {|}"
	map_2		"map f {|x:=y|} <+ A = {|x:=f(y)|} <+ (map f A)"

end
