signature BM_sig =
sig
  val single : 'a -> 'a list
  val foldl_ac : ('b -> 'a -> 'b) -> 'b -> 'a list -> 'b list
  val foldr_ac : ('a -> 'b -> 'b) -> 'b -> 'a list -> 'b list
  val inits : 'a list -> 'a list list
  val tails : 'a list -> 'a list list
  val segs : 'a list -> 'a list list
  val mss : int list -> int
  val mss2 : int list -> int
end;




structure BM_code : BM_sig =

struct

open Lib.List

fun conc a b = a@b

fun single x = [x]

fun foldl_ac f a []		= [a]
  | foldl_ac f a (x::xs)	= [a] @ (foldl_ac f (f a x) xs);

fun foldr_ac f a []		= [a]
  | foldr_ac f a (x::xs)	= f x (hd (foldr_ac f a xs)) :: foldr_ac f a xs

fun inits xs = foldl_ac conc [] (map single xs)

fun tails xs = foldr_ac conc [] (map single xs)

fun segs xs = foldl (op @) [] (map tails (inits xs))

fun mss xs = foldl max 0 (map (foldl (op +) 0) (segs xs))

fun horner a b = max ((a+b),0)

fun mss2 xs = foldl max 0 (foldl_ac horner 0 xs)



end;


(*** Testfunktionen ***)


fun intlist (n,tail) =
  if n=0 then tail
  else randlist(n-1, let val v = if n mod 2 = 0 then n else ~n in v ::tail end);


fun  test () = BM_code.mss (intlist (22,[]));
fun  test2 () = BM_code.mss2 (intlist (22,[]));