(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/tas.sml,v $
 
   TAS Main Module -- implementation part.
  
   $Date: 1997/09/12 17:18:42 $
   $Revision: 1.33 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)


functor TAS (structure trafothy : TRAFOTHEORY 
	           and conf   : TASCONF
	           and isaWin : ISAWINSIG
	    ) :TAS =

struct

    (* --- Instantiate TasGUI ------------------------------------------- *)

    structure TasGUI= TasGUI(structure conf=conf
			           and trafothy= trafothy 
			           and isa_clip= isaWin.CB) 

    open TkTypes SmlTk TasGUI Lib.List

    (* --- TAS main window ---------------------------------------------- *)


    fun cmdNotImpl() = utilWin.error "Functionality not yet implemented.\n"
	(K())
	

    fun cmdQuit(win, cc) () = 
	utilWin.confirm "Do you really want to quit?" 
	(fn ()=> (cc (TasGUI.state()); closeWindow win))

    fun cmdAbort(win, abort) () =
	utilWin.confirm "WARNING: All your work and labour will be lost \
	                \if you abort TAS.\nDo you really want that?"
	        (fn ()=> (closeWindow win; abort()))

    (* --- The filer to load signatures and specifications ------ *)

    fun textFileToObj(dir, filenm) =
	let val is= IO.open_in(dir^"/"^ filenm)
	    fun readIt str= if IO.end_of_stream is then str
			    else readIt (str^(IO.input_line is))
	    val txt= readIt ""
	in  (IO.close_in is;
	     SOME [spec(ref txt, ref filenm)])
	end handle (IO.Io _) => (utilWin.error
				 ("Can't open/read file "^filenm) (K());
				 NONE)
		 | OS.SysErr(why, _) => (utilWin.error
					  ("Can't open/read file "^filenm
					   ^":\n"^why) (K()); NONE)
    
    (* The assumption here is that examplesDir is in Isabelle's loadpath.
     *)
(*    fun name_of_theory thy  = !(hd(stamps_of_thy thy))
    fun fix_name(thy, nm)   =
	let val nmref= hd(stamps_of_thy thy)
	in  if (!nmref)= nm then ()
	    else (Debug.warning("Fixing name of theory '"^nm^"' \
	                        \(was '"^(!nmref)^"')");
		  nmref:= nm)
		  (*THIS IS DEBATABLE AND HIGHLY DANGEROUS! *)
	end
 *)

    fun thyFileToObj thynm =
	(SysDep.cd (conf.tmpPath());  (* to make sure we're in a writeable
				       * directory *)
	 let val loadedthy= trafothy.Thy.use thynm
	 in  ((* check if transformation top thy is parent of loaded theory  *)
	      if exists (fn t=> trafothy.Thy.eq(trafothy.topthy, t))
			       (trafothy.Thy.parents_of loadedthy) then
		  SOME [sign(loadedthy, ref thynm)]
	      else (utilWin.error ("Signature is not a descendant of top theory "^
				   (trafothy.Thy.name_of trafothy.topthy)) (K()); NONE)
	     )
	 end
	) handle IO.Io _ => NONE
	       | ERROR   => (utilWin.error ("Can't read signature "^thynm)
			     (K()); NONE)
	       | OS.SysErr (why, _) => (utilWin.error
					("Can't open/read file "^thynm
					 ^":\n"^why) (K()); NONE)
		   
		  
    (* this the variant used for the loader below. If that can't find
     * or read a theory, it just dies. 
     *)	    
    fun find_theory thnm = 
	trafothy.Thy.use thnm 
	handle ERROR => raise (Fail ("Can't load theory "^thnm))


    datatype item= ThyFile of string
	         | TextFile of string
 		   
    fun trim(str, len) = if (size str)> len then (substring(str, 0, len))
	                 else Lib.String.implode 
			     ((Lib.String.explode str)@
			      (map (K #" ") 
			       (BasicUtil.-- (size str, len-1))))
                               (* sorry it's too bloody hot to think
				* of something more clever here *)
	       
    structure chooser = 
	SimpleChooser(type item= item
		      fun name (ThyFile str) = trim(str, 20)^" Sign"
			| name (TextFile str)= trim(str, 20)^" Text"
		      structure CB= TasGUI.CB
		      structure UW= conf.uw
		      val selWidth= 25
		      val selHeight= 10)

    fun isSuffix(str1, str2) =
	BasicUtil.prefix (rev (explode str1)) (rev (explode str2))

    fun usefuleFile filenm=
	if (isSuffix("."^(trafothy.Thy.theoryfile_suffix), filenm)) then 
	    SOME (ThyFile (substring(filenm, 0, 
		        (size filenm- size trafothy.Thy.theoryfile_suffix-1))))
	else if isSuffix(".ML", filenm) then
	    NONE
	else if isSuffix(".ml", filenm) then
	    NONE
	else if (filenm= ".") orelse (filenm= "..") then
	    NONE		 
	else SOME (TextFile filenm)

    fun itemToObj dir (ThyFile nm)  = thyFileToObj nm
      | itemToObj dir (TextFile nm) = textFileToObj(dir, nm)
       
	   
    fun cmdFiler(win, dir)=
	let val read_me= filter (fn nm=> (SysDep.isFileRd (dir^"/"^nm))) 
		                         (SysDep.ls dir)
	    val files= mapPartial usefuleFile read_me
	    fun cont obop = case obop of 
		SOME  (obj::_) => intro (obj, conf.specsHere, conf.moreSpecs)
	      | _ => ()	   
	in  chooser.select(SOME win, "File Selection", 
			   "Please select one:", 
			   itemToObj dir, cont o itemToObj dir, files)
	end

    (* --- The Trafo chooser ------------------------------------ *)

    structure trafo_chooser= 
	SimpleChooser(type item= trafos.trafo
		      fun name (trf:trafos.trafo)= #name trf
		      structure CB= TasGUI.CB
		      structure UW= conf.uw
		      val selWidth= 20
		      val selHeight= 10)
	
    fun cmdTrafoChooser win ()= 
	let val trafos= TasGUI.trafos.all_trafos() 
	    fun trafoToObj (trf:trafos.trafo)= 
		                   SOME [trafoObj(trf, [], ref (#name trf))]
	    fun cont obop = case obop of 
		SOME  (obj::_) => intro (obj, conf.trafosHere, 
					      conf.moreTrafos)
	      | _ => ()	   
	in trafo_chooser.select(SOME win, "Trafo Selection",
			 "Please select a transformation rule",
			  trafoToObj, cont o trafoToObj, trafos)
	end


		       		       	 	
    (* --- setting of Isabelle options via the options menu ----- *)

    local
	fun toggleVar(tclVar, isaVar)= 
	    isaVar:= (readVarValue tclVar)= "1"
	
	
	val varTypes = "types"
	val varBrackets = "brackets"
	val varEta = "eta"
	    
	fun toggleTypes () =    (toggleVar(varTypes, show_types);
				 toggleVar(varTypes, show_sorts))
	fun toggleBrackets () = toggleVar(varBrackets, show_brackets)
	fun toggleEta() = toggleVar(varEta, eta_contract)

	fun initVars () = 
	    (show_types:= false; show_sorts:= false; setVarValue varTypes "0";
	     show_brackets:= false; setVarValue varBrackets "0";
	     eta_contract:= true; setVarValue varEta "1")


	(* --- The menue of the main window ------------------------- *)
				       
	fun fileMenue(win, dir, cc, exit, abort) = 
	    Menubutton(newWidgetId(), false, 
		       [MCommand[Text "Load Objects",
				 Command (fn _ => cmdFiler(win, dir))],
			MSeparator,
			MCommand[Text "Save Workspace", 
				 Command (fn _ => cc (TasGUI.state()))],
			MCommand[Text "Save & Quit", Command (cmdQuit(win, exit))],
			MCommand[Text "Abort", Command (cmdAbort(win, abort))]
		       ], [Side Left], [Text "File"],[])
	

	fun cmdEditNewSubst () =
	    enterWin.newSubst("Edit new substitution.", [""],
			      fn s=> TasGUI.intro(subst(ref s, ref "*new*"), conf.substsHere, conf.moreSubsts))

	fun cmdEditNewSpec () = 
	    (utilWin.enterText "New specification " 
	     "Please enter new specification below:"
	     "" 40 25
	     (fn t=> TasGUI.intro(spec(ref t, ref "*new spec*"), conf.specsHere, conf.moreSpecs)) (K()))


	fun devMenue win = 
	    Menubutton(newWidgetId(), false, 
		       [MCommand[Text "New Substitution", Command cmdEditNewSubst],
			MCommand[Text "New Specification", Command cmdEditNewSpec],
			MCommand[Text "Select Transformation", Command (cmdTrafoChooser win)]
		       ], [Side Left], [Text "Edit"],[])

	val optMenue =
	    Menubutton(newWidgetId(), false,
		       [MCheckbutton[Text "Full Bracketing",
				     Command toggleBrackets,
				     Variable varBrackets],
			MCheckbutton[Text "Show Sorts& Types",
				     Command toggleTypes,
				     Variable varTypes],
			MCheckbutton[Text "Eta-Contraction",
				     Command toggleEta,
				     Variable varEta]
		       ], [Side Left], [Text "Settings"], [])

	val helpMenue = 
	    Menubutton(newWidgetId(), false,
		       [MCommand[Text "About TAS", 
				 Command (fn()=> HTML.openURL "http://www.cl.cam.ac.uk/Research/HVG/isabelle.html")],
			MCommand[Text "About Isabelle", 
				 Command (fn()=> HTML.openURL "http://www.cl.cam.ac.uk/Research/HVG/isabelle.html")],
			MCommand[Text "About UniForM", 
				 Command (fn()=> HTML.openURL "http://www.informatik.uni-bremen.de/~uniform")]
			], [Side Right], [Text "Help"], [])


	fun mainMenue(win, dir, cc, exit, abort) = 
	    Frame(newWidgetId(), 
		  [fileMenue(win, dir, cc, exit, abort), devMenue win, optMenue, helpMenue],
		  [Side Top, Fill X, Expand true],
		  [Relief Raised, Borderwidth 2], [])
    in       
	fun mainWin(state, dir, cc:gui_state-> unit, exit, abort) = 
	    let val mwid = mainWinId()
		fun init () = (initVars();
			       TasGUI.init state)
	    in  (mwid, [WinTitle "UniForM Transformation Application System",
			WinGeometry (NONE, SOME conf.tasWinXY)],
		 [mainMenue(mwid, dir, cc, exit, abort), TasGUI.main_wid mwid], init)
	    end
    end

    val isaWin =
	isaWin.mainWin(newWinId(),
		       [WinTitle "UniForM Isabelle Proof Assistant",
			WinGeometry (NONE, SOME (addCoord conf.tasWinXY (500, 0)))])


(* --- Representing Objects externally ------------------------------------ *)

    type path= trafos.path

    type inst= trafos.inst

    type trafo= trafos.trafo 
	
    fun  rep (trf:trafo) = #id trf

    type history = history.ext_rep

    (* type trafoSubst = (string* string) list  -- from TasGUI *)

    datatype ext_obj =
	Dev      of string* string* history.ext_rep* string
      | TrafoObj of trafos.trafo* trafoSubst
      | Subst    of trafoSubst
      | Sign     of string 
      | Spec     of string

    type ext_obj_rep  = ext_obj* string* TkTypes.Coord
	
    type state = ext_obj_rep list
	
    fun ext2int (TrafoObj(trf, sub), nm, wh) = 
	(trafoObj(trf, sub, ref nm), wh, Center)
      | ext2int (Subst(sub), nm, wh) =
	(subst(ref sub, ref nm), wh, Center)
      | ext2int (Sign(thy), nm, wh)= 
	(sign(find_theory thy, ref nm), wh, Center)
      | ext2int (Spec(str), nm, wh)=
	(spec(ref str, ref nm), wh, Center)
      | ext2int (Dev(th, sp, hist, info), nm, wh)=
	let val thy= find_theory th
	in  (progDev(history.mk(hist, trafos.start_dev(thy, sp)), 
		     sp, ref info, ref nm), wh, Center) 	
	end

    val externalToInternal = map ext2int

    fun int2ext(trafoObj(trf, sub, nm), wh, _) =
	(TrafoObj(trf, sub), !nm, wh)
      | int2ext(subst(sub, nm), wh, _) =
	(Subst(!sub), !nm, wh)
      | int2ext(sign(th, nm), wh, _) =
	(Sign(trafothy.Thy.name_of th), !nm, wh)
      | int2ext(spec(sp, nm), wh, _) =
	(Spec(!sp), !nm, wh)
      | int2ext(progDev(hist, sp, info, nm), wh,_) =
	let val thy= trafothy.Thy.name_of(trafos.get_theory 
					     (history.cur_state hist))
	in  (Dev(thy, sp, history.rep hist, !info), !nm, wh)
	end

    val internalToExternal = map int2ext

    val external_dir = ref (conf.examplesPath())
        
    fun go(state, save, exit, abort) =  
	ignore (SmlTk.startTcl[mainWin(externalToInternal state, 
				      !external_dir, 
				      save o internalToExternal, 
				      exit o save o internalToExternal, abort), 
			       isaWin])
	handle TasGUI.GenGUI err => raise Fail ("GenGUI error: "^err)    

    val initial_state = internalToExternal o TasGUI.initial_state

end

