(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/tas_csp.sml,v $
 
   TAS instantiation for the Communicating Sequential Processes.

   This assumes CSP is already loaded. If it isn't, do "use_thy "Csp"
   and be prepared to wait about two hours.

   $Date: 1997/09/19 14:36:53 $
   $Revision: 1.12 $
   Author: cxl (Last modification by $Author: cxl $)

   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen

  ************************************************************************** *)

(* Put trafo theories in loadpath *)
val thydir=   "/home/cxl/src/isa_lang/CSP/univ_csp";
val trafodir= "/home/cxl/src/isa_lang/CSP/univ_csp/examples";
val examples= tasdir^"/examples/CSP";
 
let
    fun addel(e, l) = if exists (fn ee=>ee= e) l then l else e:: l
in                  
    loadpath := addel(examples, addel(trafodir, addel(thydir, !loadpath)))
end;


(* Load top theory *)
use_thy "CspTrafos"; 
SGML.attach_ext_syntax CspTrafos.thy [] [];

SGML.attach_ext_syntax Copy.thy [] [];

(* Load trafos *)
use "trafos/std_thy.sml";
use "trafos/un_fold.sml";
use "trafos/csp_trafos.sml";

(* Load configuration *)
use "config.sml";

(* --- IsaWin for CSP ----------------------------------------------------- *)

use_thy "Exa1";
SGML.attach_ext_syntax Exa1.thy [] [];

val huge_ss = merge_ss(set_ss,prod_ss)
                addsimps([Un_def,subset_spec,empty_spec,
                          Collect_eq_spec, Int_def,SYN_def]@channel.simps)
                addcongs [conj_cong];

val Copy_ss = 
            merge_ss(list_ss,Alph_ss) addsimps 
	    [renamF_cont,RenRecPref,RenRecLpref,RenFPref,RenFLpref]	
	    @Copy.chan.simps;

(*
evtl auf die flaeche:
mono_mprefix_ref
hide_prefix_distr1
mprefix_Par_Int_distr3
hide_mprefix_distr
mprefix_singl
mprefix_Par_Int_distr1
singleton_spec
conj_commute
*)

structure IsaCSP = 
    IsaWin(open IsaConfNormal
    	   open IsaWinTypes

	   val initConf = 
	       {thms =   [(SYSTEM_def, rewrRule, "SYSTEM_def"),
			  (COPY_def,   rewrRule, "COPY_def"), 
			  (Let_def,    rewrRule, "Let_def"),
			  (Letrec_def, rewrRule, "Letrec_def"),
			  (fix_ind,    introThm, "fix_ind")],
	       clasets=  [(eq_cs, "eq_ss")],
	       simpsets= [(HOL_ss, "HOL_ss"),
			  (cont_ss, "cont_ss"),
			  (UU_ss,   "UU_ss"),
			  (Copy_ss, "Alphabet_ss"),
			  (huge_ss, "setE_ss")],
	       theories= [(Csp.thy, "Csp"), (Copy.thy, "Copy"),
			  (Exa1.thy, "Exa1")],
	       texts=    [("(COPY::'a channel process) << SYSTEM",
			   "refinement proof")],
	       goals=    [],
	       substs=   [([("f","(%(SEND,REC). (?X SEND, ?Y REC))")], 
								"fix_redex")],
	       eta_contract= false}
	   )
	    


(* --- Instantiate TAS ---------------------------------------------------- *)
structure TasCSP= 
    TasAPI(structure TAS= TAS(structure trafothy = CspTrafoThy
			      structure conf =    TasConfNormal
				                     (val examples_dir= 
							  "examples/CSP")
			      structure isaWin = IsaCSP
			    ));
(* Open TAS *)
open TasCSP

(* --- Example States ----------------------------------------------------- *)

val Ex1 = 
[(TrafoObj(CspTrafoThy.R1pref_trafo,[]),"Rule #1",(318,11)),
 (TrafoObj(CspTrafoThy.COPY2_def,[]),"COPY2_def",(236,75)),
 (TrafoObj(CspTrafoThy.COPY1_def,[]),"COPY1_def",(237,12)),
 (Spec("COPY1 ||| COPY2 <<?x\n\n\n"),"*new spec*",(80,10)),
 (Dev("CspTrafos","COPY1 ||| COPY2 <<?x\n\n\n",(0,[]),""),"*new spec*",(10,80)),
 (Sign("CspTrafos"),"CspTransfo1",(10,10))];

val Ex2=

[(Dev("CspTrafos","COPY1 ||| COPY2 <<?x\n\n\n",(4,[opApplyTrafoAt(CspTrafoThy.COPY1_def,[],"210"),opApplyTrafoAt(CspTrafoThy.COPY2_def,[],"20"),opApplyTrafoAt(CspTrafoThy.R1pref_trafo,["mid1","ack1"],"210"),opApplyTrafoAt(CspTrafoThy.R1pref_trafo,["mid2","ack2"],"20")]),""),"*new spec*",(14,79)),
 (TrafoObj(CspTrafoThy.R1pref_trafo,[("m","mid2"),("a","ack2")]),"Rule #1",(318,151)),
 (Subst([("m","mid2"),("a","ack2")]),"*new*",(430,10)),
 (TrafoObj(CspTrafoThy.R1pref_trafo,[("m","mid1"),("a","ack1")]),"Rule #1",(318,81)),
 (Subst([("m","mid1"),("a","ack1")]),"*new*",(500,10)),
 (TrafoObj(CspTrafoThy.COPY2_def,[]),"COPY2_def",(236,75)),
 (TrafoObj(CspTrafoThy.COPY1_def,[]),"COPY1_def",(237,12)),
 (Spec("COPY1 ||| COPY2 <<?x\n\n\n"),"*new spec*",(80,10)),
 (TrafoObj(CspTrafoThy.R1pref_trafo,[]),"Rule #1",(318,11)),
 (Sign("CspTrafos"),"CspTrafos",(10,10))];

val BigEx = 

[(Dev("CspTrafos","COPY1 ||| COPY2 <<?x\n\n\n",(8,[opApplyTrafoAt(CspTrafoThy.COPY1_def,[],"210"),opApplyTrafoAt(CspTrafoThy.COPY2_def,[],"20"),opApplyTrafoAt(CspTrafoThy.R1pref_trafo,["mid1","ack1"],"210"),opApplyTrafoAt(CspTrafoThy.R1pref_trafo,["mid2","ack2"],"20"),opApplyTrafo(CspTrafoThy.R5b_trafo,[]),opApplyTrafoAt(CspTrafoThy.R6_trafo,[],"210"),opApplyTrafoAt(CspTrafoThy.R3_trafo,["c1","c2","mid1","mid2"],"211210"),opApplyTrafoAt(CspTrafoThy.R3_trafo,["a1","a2","ack1","ack2"],"21121211210")]),""),"COPY dev.",(14,79)),
 (TrafoObj(CspTrafoThy.R3_trafo,[("a1","ack1"),("a2","ack2"),("c1","a1"),("c2","a2")]),"Channel Intro",(381,288)),
 (Subst([("a1","ack1"),("a2","ack2"),("c1","a1"),("c2","a2")]),"Chan.Intr.2",(152,83)),
 (TrafoObj(CspTrafoThy.R3_trafo,[("a1","mid1"),("a2","mid2"),("c1","c1"),("c2","c2")]),"Channel Intro",(308,290)),
 (Subst([("m","mid1"),("a","ack1")]),"*new*",(493,7)),
 (Subst([("m","mid2"),("a","ack2")]),"*new*",(425,7)),
 (Subst([("a1","mid1"),("a2","mid2"),("c1","c1"),("c2","c2")]),"Chan.Intr.1",(152,12)),
 (TrafoObj(CspTrafoThy.R3_trafo,[]),"Channel Intro",(351,226)),
 (TrafoObj(CspTrafoThy.R6_trafo,[]),"Sync_Inter_Dist",(217,250)),
 (TrafoObj(CspTrafoThy.R5b_trafo,[]),"Hide",(101,250)),
 (Sign("CspTrafos"),"CspTrafos",(10,10)),
 (TrafoObj(CspTrafoThy.R1pref_trafo,[]),"Rule #1",(318,11)),
 (Spec("COPY1 ||| COPY2 <<?x\n\n\n"),"COPY spec",(80,10)),
 (TrafoObj(CspTrafoThy.COPY1_def,[]),"COPY1_def",(237,12)),
 (TrafoObj(CspTrafoThy.COPY2_def,[]),"COPY2_def",(236,75)),
 (TrafoObj(CspTrafoThy.R1pref_trafo,[("m","mid1"),("a","ack1")]),"Rule #1",(318,81)),
 (TrafoObj(CspTrafoThy.R1pref_trafo,[("m","mid2"),("a","ack2")]),"Rule #1",(318,151))];

val Demo =

[(Dev("CspTrafos","COPY1 ||| COPY2 <<?x\n\n\n",(9,[opApplyTrafoAt(CspTrafoThy.COPY1_def,[],"210"),opApplyTrafoAt(CspTrafoThy.COPY2_def,[],"20"),opApplyTrafoAt(CspTrafoThy.R1pref_trafo,["mid1","ack1"],"210"),opApplyTrafoAt(CspTrafoThy.R1pref_trafo,["mid2","ack2"],"20"),opApplyTrafo(CspTrafoThy.R5b_trafo,[]),opApplyTrafoAt(CspTrafoThy.R6_trafo,[],"210"),opApplyTrafoAt(CspTrafoThy.R3_trafo,["c1","c2","mid1","mid2"],"211210"),opApplyTrafo(CspTrafoThy.Renaming_trafo,[])]),""),"COPY dev.",(14,79)),
 (TrafoObj(CspTrafoThy.Renaming_trafo,[]),"Rename",(467,149)),
 (Spec("\n(/ COPY. left1 -> right1 -> COPY) ||| (/ COPY. left2 -> right2 -> COPY) << ?x\n\n\n\n\n\n"),"COPY unfolded def.",(80,78)),
 (TrafoObj(CspTrafoThy.R5b_trafo,[]),"Hide",(467,18)),
 (TrafoObj(CspTrafoThy.R6_trafo,[]),"Sync_Inter_Dist",(469,83)),
 (Subst([("m","mid2"),("a","ack2")]),"*new*",(340,80)),
 (TrafoObj(CspTrafoThy.R1pref_trafo,[("m","mid2"),("a","ack2")]),"Buffer intro, right",(266,78)),
 (TrafoObj(CspTrafoThy.R1pref_trafo,[]),"Rule #1",(264,12)),
 (TrafoObj(CspTrafoThy.COPY2_def,[]),"COPY2_def",(192,75)),
 (TrafoObj(CspTrafoThy.COPY1_def,[]),"COPY1_def",(193,12)),
 (TrafoObj(CspTrafoThy.R3_trafo,[("a1","mid1"),("a2","mid2"),("c1","c1"),("c2","c2")]),"Channel Intro",(646,77)),
 (Subst([("a1","mid1"),("a2","mid2"),("c1","c1"),("c2","c2")]),"Chan.Intr.1",(579,77)),
 (Subst([("a1","ack1"),("a2","ack2"),("c1","a1"),("c2","a2")]),"Chan.Intr.2",(580,138)),
 (TrafoObj(CspTrafoThy.R3_trafo,[("a1","ack1"),("a2","ack2"),("c1","a1"),("c2","a2")]),"Channel Intro",(648,137)),
 (TrafoObj(CspTrafoThy.R3_trafo,[]),"Channel Intro",(643,12)),
 (Spec("COPY1 ||| COPY2 <<?x\n\n\n\n\n"),"COPY spec",(81,12)),
 (Sign("CspTrafos"),"CspTrafos",(10,10))];




val CSPDemo =

[(TrafoObj(CspTrafoThy.R1pref_trafo,[]),"BufferIntro",(551,80)),
 (TrafoObj(CspTrafoThy.R6_trafo,[]),"SyncInterDist",(267,77)),
 (TrafoObj(CspTrafoThy.R5b_trafo,[]),"Hide",(197,77)),
 (TrafoObj(CspTrafoThy.R3_trafo,[]),"ChannelIntro",(411,80)),
 (Subst([("a1","ack1"),("a2","ack2"),("c1","a1"),("c2","a2")]),"Chan.Intr.2",(483,12)),
 (Subst([("a1","mid1"),("a2","mid2"),("c1","c1"),("c2","c2")]),"Chan.Intr.1",(412,12)),
 (TrafoObj(CspTrafoThy.COPY2_def,[]),"CP2_def",(270,10)),
 (TrafoObj(CspTrafoThy.COPY1_def,[]),"CP1_def",(198,11)),
 (Spec("COPY1 ||| COPY2 <<?x\n\n\n"),"CP_spec",(80,10)),
 (Sign("CspTrafos"),"CspTrafos",(10,10)),
 (Subst([("m","mid2"),("a","ack2")]),"BufferIntro2",(624,10)),
 (Subst([("m","mid1"),("a","ack1")]),"BufferIntro1",(553,12))];
(* --- Message ------------------------------------------------------------- *)

writeln "\n\n\
\TAS/CSP is now set up.\n\
\Use start(); to start TAS with its initial state, and resume <state>; to \n\
\resume execution. You may also want to try resuming the example states\n\
\Ex1, Ex2 and BigEx.\n";
