(* ***************************************************************************
 
   $Source: /home/uniform/rep/isa_gui/tas/tas_hol.sml,v $
 
   TAS instantiation for Common Higher-Order Logic. 

   $Date: 1997/09/12 14:31:59 $
   $Revision: 1.5 $
   Author: cxl (Last modification by $Author: tm $)

   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen

  ************************************************************************** *)

(* Put trafo theories in loadpath *)
val trafodir= tasdir^"/../../isa/isa_toolkit/trafos";
val morethys= tasdir^"/../../isa/isa_toolkit/thy";

(*  Where to find examples: *)
val examples= tasdir^"/examples/HOL";
 
let
    fun addel(e, l) = if exists (fn ee=>ee= e) l then l else e:: l
in                  
    loadpath := addel(examples, addel(morethys, addel(trafodir, !loadpath)))
end;


(*Load top theory *)
use_thy "TAS_HOL0";

(* Load trafos *)
use "trafos/std_thy.sml";
use "trafos/un_fold.sml";
use "trafos/hol_trafos.sml";

(* Load configuration *)
use "config.sml";

(* --- Instantiate TAS ---------------------------------------------------- *)
structure TasHOL= 
    TasAPI(structure TAS= TAS(structure trafothy = HolTrafoThy
			      structure conf=    TasConfNormal
				                     (val examples_dir= 
							  "examples/HOL")
			      structure isaWin = IsaWinHOL(IsaConfNormal)  
			    ));

(* Open TAS *)
open TasHOL


(* --- Example States ----------------------------------------------------- *)

val Ex1=
[(Sign("TAS_HOL"),"TAS_HOL",(10,10)),
 (TrafoObj(HolTrafoThy.SoPC_trafo,[]),"SoPC",(233,12)),
 (TrafoObj(HolTrafoThy.gs_trafo,[]),"GlobalSearch",(232,83)),
 (TrafoObj(FoldTrafos.unfold_trafo,[]),"Unfold",(344,10)),
 (TrafoObj(FoldTrafos.fold_trafo,[]),"Fold",(416,11)),
 (TrafoObj(HolTrafoThy.commuteAC_trafo,[]),"Commute (AC)",(418,82)),
 (TrafoObj(HolTrafoThy.commute_trafo,[]),"Commute",(343,81)),
 (Sign("Fmap"),"Fmap",(25,79)),
 (Spec("((% (U,V). U:Fin(U) & V:Fin(V))(A,B) --> (fin_maps (A,B) M = (% (U,V) N. N : Map(U,V)) (A,B) M)) = ?X\n"),"fmap.spec",(27,148)),
 (Dev("Fmap","((% (U,V). U:Fin(U) & V:Fin(V))(A,B) --> (fin_maps (A,B) M = (% (U,V) N. N : Map(U,V)) (A,B) M)) = ?X\n",(0,[]),""),"fmap.spec",(99,151)),
 (Subst([("Ih", "% (U,V) (S,T,M). S Un T = U & S Int T = {} & M:Map(S,V)"),
	("r0", "% (U,V). ({}, U,{|})"),
	("Split", "(%(U,V) (S,T,M) (S',T',M').(EX a b. (a:T) & (b:(V::'b set)) & ((S',T',M') = ((S :: 'a set) Un {a},(T:: 'a set) - {a}, M <+ {|a := b|}))))"),
	("Extract", "% N (S,T,M). T ={} & N = M"),
	("subspace", "space(% (S,T,M) (S',T',M'). S <= S' & S ~= S')"),
	("Satisfies", "% N (S,T,M). (ALL x. x : S--> N^x = M^x)")]), 
 "GS Subst", (11,58))];


val HOLDemo =
[(Spec("(a::nat)+c*b=?X\n\n"),"Arith2.spec",(143,77)),
 (TrafoObj(HolTrafoThy.commute_trafo,[]),"Commute",(416,82)),
 (Spec("(a::nat) * (c + b) = ?X\n"),"Arith.spec",(144,10)),
 (TrafoObj(HolTrafoThy.add_mult_trafo,[]),"Distribute",(487,82)),
 (TrafoObj(HolTrafoThy.commute_plus_trafo,[]),"CommutePlus",(346,82)),
 (Spec("(!k n.k = sqrt n --> sqr k <= n & ~sqr (succ k) <= n) = ?X\n"),"Sqrt.spec",(220,9)),
 (TrafoObj(HolTrafoThy.SoPC_trafo,[]),"SoPC",(556,82)),
 (Subst([("I","%n k. sqr k <= n"),("B","%n k. ~(sqr(succ k) <= n)"),("E","%n. zero"),("H","%n k. succ k")]),"SoPC_Subst",(557,12)),
 (Sign("TAS_HOL0"),"TAS_HOL0",(16,9))];


(* --- Message ------------------------------------------------------------- *)

writeln "\n\n\
\TAS for Higher Order Logic is now setup.\n\
\Use start(); to start TAS with its initial state, and resume <state>; to \n\
\resume execution. You may also want to try resuming the example state Ex1.\n";
