(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/njml.sml,v $
 
   Implementation of system dependent functions for SMLNJ 093.
  
   $Date: 1997/11/14 10:42:53 $
   $Revision: 1.12 $
   Author: stefan (Last modification by $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

structure SysDep : SYS_DEP
=
struct

    type 'a option = 'a Bool.option

    val cd      = System.Directory.cd
    val ls      = System.Directory.listDir
    val pwd     = System.Directory.getWD
    val isDir   = System.Directory.isDir
    val execute = IO.execute
    val exec s sl = true

    val getEnv  = UnixEnv.getEnv

    fun isFileRdAndEx pn =
	let 
	    val {dir=d,ext=e,name=n} = Pathname.splitPath pn
	    val dir  = d
	    val name = if ( e = "") then n
		       else n^"."^e
	in
	    ((UnixPath.findFileOfType 
	        (Pathname.mkSearchPath dir,
		 UnixPath.F_REGULAR,
		 [UnixPath.A_READ,UnixPath.A_EXEC])
		name;
		true)
	     handle UnixPath.NoSuchFile => false ) orelse

	    ((UnixPath.findFileOfType 
	        (Pathname.mkSearchPath dir,
		 UnixPath.F_SYMLINK,
		 [UnixPath.A_READ,UnixPath.A_EXEC])
		name;
		true)
	     handle UnixPath.NoSuchFile => false )
	end

    fun isFileRd pn =
	let 
	    val {dir=d,ext=e,name=n} = Pathname.splitPath pn
	    val dir  = d
	    val name = if ( e = "") then n
		       else n^"."^e
	in
	    ((UnixPath.findFileOfType 
	        (Pathname.mkSearchPath dir,
		 UnixPath.F_REGULAR,
		 [UnixPath.A_READ])
		name;
		true)
	     handle UnixPath.NoSuchFile => false ) orelse

	    ((UnixPath.findFileOfType 
	        (Pathname.mkSearchPath dir,
		 UnixPath.F_SYMLINK,
		 [UnixPath.A_READ])
		name;
		true)
	     handle UnixPath.NoSuchFile => false )
	end

    fun isDirRdAndWr pn =
	let 
	    val dir = Pathname.dirOfPath
	    val name = Pathname.nameOfPath
	in
	    (UnixPath.findFileOfType 
	       (Pathname.mkSearchPath (dir pn),
		UnixPath.F_DIR,
		[UnixPath.A_READ,UnixPath.A_WRITE]) 
	       (name pn);
             true)
	     handle UnixPath.NoSuchFile => false
	end

    fun openFile e f = open_in(f) handle Io str => raise (e str)

    val inc     = inc;

    val nth     = nth;

    fun isPrint c     = c >= " "   andalso  c <= "~"
    fun isUpper c     = c >= "A"   andalso  c <= "Z"
    fun isLower c     = c >= "a"   andalso  c <= "z"

    fun is_letter ch =
       ord "A" <= ord ch andalso ord ch <= ord "Z" orelse
       ord "a" <= ord ch andalso ord ch <= ord "z"

    fun is_digit ch =
       ord "0" <= ord ch andalso ord ch <= ord "9"

    fun isAlpha c     =  is_letter c
    fun isDigit c     =  is_digit c
    fun isAlphanum c  =  isAlpha c  orelse  isDigit c
    fun isSpace c     =  c = " "   orelse c = "\t"  orelse c = "\n"


    val isDot        = fn s => "." = s
    val isComma      = fn s => "," = s
    val isLinefeed   = fn s => "\n" = s
    val isOpenParen  = fn s => "(" = s
    val isCloseParen = fn s => ")" = s
    
    val numval = 
        let
           fun foldl f z []      = z
               |foldl f z (x::xs) = foldl f (f z x) xs
        in
           foldl (fn n => fn x => 10 * n + ((ord x) - (ord "0"))) 0
        end;

    (* adaptstring takes care of double quotes and maybe other 	*)
    (* special characters,					*)
    (* to insure that Tcl/Tk does get the right string.		*)
    (* adaptstring :: string -> string				*)
    fun adaptstringH [] = ""
      | adaptstringH (x::xs) = 
	if exists (fn y => x=y) (explode "\"\\$[]{}")
	    then "\\" ^ x ^ adaptstringH xs
        else if x = "\n" then "\\n" ^ adaptstringH xs
             else x ^ adaptstringH xs

    val adaptString = fn s => adaptstringH(explode s)

    fun min x y = Integer.min(x,y)
    fun max x y = Integer.max(x,y)

    (* aus Isabelle ... *)
    fun xSmlTk filename banner = (exportML filename;  print(banner^"\n"))

    fun setPrintDepth n = (System.Control.Print.printDepth := n div 2;
			   System.Control.Print.printLength := n)

end;


