
(* *********************************************************************** *)
(*									   *)
(* Project: sml/Tk: an Tk Toolkit for sml	 			   *)
(* Author: Burkhart Wolff, University of Bremen	 			   *)
(* Date: 25.7.95				 			   *)
(* Purpose of this file: Small example					   *)
(*									   *)
(* *********************************************************************** *)

structure Ex2 =
struct

local open SysDep in
local open BasicTypes in
local open SmlTk in

exception NoFile of string;

fun prs s = output (std_out, s);
fun writeln s = prs (s ^ "\n");

val noaction = fn () => ();

(* Cursor Get Action on Listboxes and TextWidgets *)

fun getcur wid = fn (_)=> let val Mark(n,m) = readCursor wid
                          in  output(std_out, "POSITION :" ^ makestring n ^
						"."^ makestring m ^"\n") 
                          end;


(* Warning Window *)

val nogoon   = fn () => closeWindow "warning";

fun nobut   msg yes = Button("no",[Side Right, Fill X, Expand true],
			[Text "NO",  Command  nogoon],[]);

fun message msg yes = Label("l1",[Fill X, Expand true],
				[Text msg, Relief Flat, Width 25],[]);
fun yesbut  msg yes = Button("yes",[Side Left,  Fill X, Expand true],
					[Text "YES", Command yes],[]);
fun yesno   msg yes = Frame("yn",[yesbut msg yes, nobut msg yes],
				[],[],[]);

fun tree2   msg yes = [message msg yes, yesno msg yes];


fun warnwin msg yes = ("warning", 
		       [WinTitle "Warning",WinTransient (SOME "main")], 
		       tree2 msg yes,noaction);

fun warning msg yes = openWindow (warnwin msg yes);



(* Enter Window *)

val nlabel   = Label("l2",[Side Left],[Text "name:"],[]);

fun input enteraction = 
     let val mrs = fn (_) => let val nm = readTextAll "e1"
		            in  enteraction nm ();closeWindow "entername" end 
     in Entry("e1",[],[Width 20], [Bind("<Return>",mrs)]) end;

fun treesize enteraction = [nlabel,input enteraction];

fun enterwin enteraction = ("entername", 
			    [WinTitle "Please enter name",WinTransient (SOME "main")], 
			    treesize enteraction,noaction);



(* Main Window *)

val yesquit  = fn () => closeWindow "main"; 

val entername = 
     let fun inputok nm = fn () => insertTextEnd "liste" nm
         val cmd        = fn () => openWindow (enterwin inputok)
     in MCommand [Text "Enter name", Command cmd] end;

val m1 = newWidgetId();
val insertmenue= Menubutton(m1,true,[entername],[Side Left],[Text "Special"],[]);

val list     = Listbox("liste",RightScb,
			[Side Left,Side Top,Fill Y],[Relief Raised], 		
			[Bind("<Double-Button-1>",getcur "liste")]);

val MARKLIST = ref([] : (Mark * Mark) list);

val statewid = 
    TextWid("statewid",RightScb,"",[],[Side Left,Fill Both],
	    [Relief Raised, Borderwidth 2],
	    [Bind("<Double-Button-1>",getcur "statewid"),
	     Bind("<Double-Button-3>",
		  fn (_) => let val t = readSelWindow();
				val m = readSelRange "statewid";
			    in
				case t of
				    NONE => 
					writeln("SEL")
				  | SOME (win,wid) => 
					writeln(win ^ wid);
					MARKLIST := m
			    end),
	     Bind("<Double-Button-2>",
		  fn (_) => output(std_out, readTextAll 
				   "statewid"))]);

val yesreset = noaction;

val loadfile = 
     let fun more str = let val in_str = SysDep.openFile NoFile str
    		        in  while not(end_of_stream in_str) do
			      insertTextEnd "statewid" (inputc in_str 100)
    		        end;
         fun doit nm = fn () => (clearText "statewid";more nm);
     in  fn () => (openWindow (enterwin doit)) 
     end

val load     = MCommand[Text "Load", Command loadfile];

val quitgame = fn () => warning "really quit?"  yesquit;

val quit     = MCommand([Text "Quit", Command quitgame]); 

val m2 = newWidgetId();
val ctrmenue = Menubutton(m2,true,[load, MSeparator, quit],[Side Left],
		[Text "Control"],[]);

val space    = Frame("space",[],[Side Left, Fill Both],[],[]);
val menu     = Frame("menu",[ctrmenue,insertmenue,space],[],[],[]);

val board    = Frame("fr1",[statewid,list],[Side Bottom,Fill Both],[], []);

val tree1    = [menu,board];

val initwin  = [("main", [WinTitle "Transformation System"], tree1,noaction)];



(* Launching and Shutting System *)
(*
openWindow (hd initwin); 

start_tcl initwin;
start_tcl_exn initwin;

Cursor-Position fuer textwidgets:
.text index insert

Cursor-Position fuer listboxes:
 bind .dateien <Double-Button-1> {
   puts [.dateien curselection]
}


*)

val doit = fn () => startTclExn initwin

end; end; end;

end;
