
(*
 *
 *  Project: sml/Tk: an Tk Toolkit for sml
 *  Author: Stefan Westmeier, University of Bremen
 * $Date: 1997/03/26 16:16:50 $
 * $Revision: 1.8 $
 *  Purpose of this file: Tag example
 *
 *)

structure TagEx :
    sig val go : unit -> string end =
struct

open TkTypes SmlTk BasicUtil


(* Main Window *)


val menu     = 
    let
	val quit      = fn () => closeWindow (mainWinId())

	fun delBut tn = fn () => delAnnotation "t1" tn

	fun delTag tn = fn (_:TkEvent) => delAnnotation "t1" tn

	fun colTag tn co = fn (_:TkEvent) => addAnnotationConf 
	                                     "t1" tn [Background co]

	val newBut = fn () => 
                     let 
			 val tn = newAnnotationId()
		     in
			 TAWidget(tn,MarkEnd,newAnnotationFrameId(),
				  [Button(newWidgetId(),[Fill X],
					  [Text "Delete Me",
					   Command (delBut tn)],
					  [])],
				  [],
				  [],
				  [])
		     end

	val newTag = fn () => 
                     let 
			 val tn = newAnnotationId()
		     in
			 TATag(tn,
			       [(Mark(2,13),Mark(2,21))],
			       [Background Blue,Borderwidth 2,Relief Raised],
			       [Bind("<ButtonPress>",delTag tn),
				Bind("<Enter>",colTag tn Red),
				Bind("<Leave>",colTag tn Blue)])
		     end

	val newSel = fn () => 
                     let 
			 val tn = newAnnotationId()
		     in
			 TATag(tn,
			       readSelection "t1",
			       [Background Green,Borderwidth 2,Relief Raised],
			       [Bind("<ButtonPress>",delTag tn),
				Bind("<Enter>",colTag tn Red),
				Bind("<Leave>",colTag tn Green)])
		     end

	val addBut = fn () => addAnnotation
	                        "t1"
				(newBut())

	val addTag = fn () => addAnnotation
	                        "t1"
				(newTag())

	val addSel = fn () => addAnnotation
	                        "t1"
				(newSel())

	fun printTags wid = fn () =>
	    let
		val widg = getWidget wid
		val ans  = selTextWidAnnotations widg
		val ans' = filter 
		           (fn an => (Annotation.selAnnotationType an = BasicTypes.ATTag)) 
			   ans

		fun prtAnPos an =
		    let
			val tn = selAnnotationId an
			val ms = readAnnotationMarks wid tn
		    in
			output(std_err,"Tag: "^tn^"\n");
			output(std_err,"\t"^(Mark.showL ms)^"\n")
		    end
	    in
		app prtAnPos ans'
	    end


    in
	Frame("menu",
	      [Menubutton(newWidgetId(),true,
			  [MCommand([Text "Quit",Command quit])
			   ],
			  [Side Left],
			  [Text "File"],
			  []),
	       Menubutton("itemmenu",true,
			  [MCommand([Text "Add Button",Command addBut]),
			   MCommand([Text "Add Tag"   ,Command addTag]),
			   MCommand([Text "Conv Sel"  ,Command addSel]),
			   MCommand([Text "Clear Text", 
				     Command (fn ()=> clearText "t1")]),
			   MCommand([Text "Insert New Text", 
				     Command  (fn ()=> insertTextEnd "t1" 
				             ("No never, no never no more\n"^
					      "will I trust the Elves of Dunsinore\n"))]),
			   MSeparator,
			   MCommand([Text "Print Tags",Command (printTags "t1")])
			   ],
			  [Side Left],
			  [Text "Item"],
			  []),
	       Menubutton(newWidgetId(), true,
			  [MCheckbutton [Text "Writeable",
					 Variable "TWState",
					 Command (fn ()=>
					  case(readVarValue "TWState") of
					      "0" => setTextWidReadOnly "t1" true
					     | _  => setTextWidReadOnly "t1" false
					 )],
			   MSeparator,
			   MCommand([Text "Clear Text+Annotations", 
				     Command (fn ()=> clearTextWidText "t1")]),
			   MCommand([Text "Replace Text+Annotations", 
				     Command (fn ()=> let
					                 val t  = "Neuer Text\n"
					                 val tg = TATag("fat",[(Mark(1,0),Mark(1,5))],
									[Background Red,Borderwidth 2,Relief Raised],[])
						      in
							  replaceTextWidText "t1" t [tg]
						      end
						  )])
			   ],
			  [Side Left], [Text "Widget State"], [])
	       ],
	      [Fill X],
	      [Relief Raised,Borderwidth 2],[])
    end

val board    = 
    let
	val t  = "\nDies ist ein Tag-Test\n\nUnd noch ein Test ...\n"
	val tg = TATag("fat",[(Mark(2,9),Mark(2,21))],
		       [Background Red,Borderwidth 2,Relief Raised],
		       [Bind("<ButtonPress>", fn _ => 
			     (output(std_out, "Button press in annotation\n");
			      addAnnotationConf "t1" "fat" [Background Blue])),
			Bind("<ButtonRelease>", fn _ => 
			     (output(std_out, "Button release in annotation\n");
			      addAnnotationConf "t1" "fat" [Background Red])),
			Bind("<Enter>", fn _ => 
			     output(std_out, "Annotation entered\n"))])
	val wg1 = TAWidget("win1",Mark(3,0),newAnnotationFrameId(),
			   [Button("but11",[Fill X],
				   [Text "Push Me",
				    Command (fn () => () )],
				   []),
			    Button("but12",[Fill X],
				   [Text "Push Me",
				    Command (fn () => () )],
				   [])
			    ],
			   [],
			   [],
			   [])
	val wg2 = TAWidget("win2",Mark(3,0),newAnnotationFrameId(),
			   [Button("but2",[Fill X],
				   [Text "Push Me",
				    Command (fn () => () )],
				   [])],
			   [],
			   [],
			   [])
    in
	Frame("fr1",
	      [TextWid("t1",LeftScb,
		       t,
		       [tg,wg1,wg2],
		       [],
		       [TextWidReadOnly true],
		       [])
	       ],
	      [Side Left, Fill X],
	      [Width 200,Height 200,Relief Raised,Borderwidth 2], [])
    end

val area    = [menu,board];

val initwin  = [(mainWinId(), [WinTitle "Tag Example"], area,fn () => ())];


val go = fn () => startTclExn initwin


end;

