(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/toolkit/filer.sml,v $
 
   Generic file and directory selection.
 
   $Date: 1997/04/23 14:04:00 $
   $Revision: 1.19 $
   Authors: bu, kol (Last modification $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems (BISS), Universitaet Bremen
 
  ************************************************************************** *)

signature FILER_SIG =

sig
    (* stand alone versions: *)
    val file_select : unit -> string * string 
    val dir_select  : unit -> string
    (* system versions: *)
    val enter_file : (string * string -> unit) -> unit
    val enter_dir  : (string -> unit) -> unit
end


(* Filer with a clipboard and a generic regular expression matcher *)

(* There are more readily usable instantiations of the filer below,
 * near the end of this file.
 *)

functor RegExClipFiler
    (structure M : 
	 sig val filter_files : string -> string -> bool;
	     exception bad_regexp
	     structure CB: CLIPBOARD_W
	     val fileToObj : string* string-> CB.obj
	     val dirToObj  : string-> CB.obj
	 end): FILER_SIG
 =

struct

local open SysDep BasicUtil BasicTypes SmlTk ErrWarnWindows in


val CUR_DIR = ref(".");
val EXT_CUR_DIR = ref(".");
val CUR_FILE_FILTER = ref("");
val SEL_FILE_DIR = ref("");
val SEL_FILE_FIL = ref("");
val DIRS = ref([]:string list); 
val FILES = ref([]:string list); 


fun filter_pair P [] = ([],[])
   |filter_pair P (a::R) = 
	let val (b,c) = filter_pair P R
        in  if P a then (a::b, c) else (b,a::c) end;


fun showDirAndFiles () = 
let val (dirs,files) = filter_pair isDir (ls (!CUR_DIR))
in  	clearText "direntry";    
	insertTextEnd "direntry" (!CUR_DIR);
	clearText "filterentry";   
	insertTextEnd "filterentry" (!CUR_FILE_FILTER);
	clearText "fileentry";   
	insertTextEnd "fileentry" (!SEL_FILE_FIL);
        DIRS := (sort (op <=) dirs);
	FILES:= let val all = if size(!CUR_FILE_FILTER)= 0 then files
			      else ((filter (M.filter_files (!CUR_FILE_FILTER))
				     files) 
				     handle bad_regexp=>
					(CUR_FILE_FILTER:= "";
					 errs_occurred1
					 "Bad regular expression"();
					 files)
				    )
		in (sort (op <=) all)
		end;
	clearText "dirliste";
	app (insertTextEnd "dirliste") (!DIRS); 
	clearText "fileliste";
	app (insertTextEnd "fileliste") (!FILES) 
	(* functional programming what's it like eh? *)
end;

fun showDir () = 
let val (dirs,files) = filter_pair isDir (ls (!CUR_DIR))
in  	clearText "direntry";   
	insertTextEnd "direntry" (!CUR_DIR);
        DIRS := dirs;
	clearText "dirliste";
	app (insertTextEnd "dirliste") (!DIRS) 
end;


fun build_dir_list win sw = 
   let fun cmd (_:TkEvent) = 
       let val Mark(n,_) = readCursor "dirliste"
           val _ = Debug.print 12 ("FILER n= "^(makestring n)^"\n");
	   val newdir = nth(!DIRS,n)
       in  (cd newdir;  (* Provisory; may fail *)
	    CUR_DIR:= pwd(); 
	    if sw then () else SEL_FILE_DIR:= (!CUR_DIR);
		if sw then showDirAndFiles() else showDir ())
	   handle NotDirectory => 
	       errs_occurred1 "Permission denied" ()
       end;
       fun cmd1 (e:TkEvent)= (cmd(e);cd (!EXT_CUR_DIR); closeWindow win)
   in  Listbox("dirliste",RightScb,
			(if sw then [Side Left] else []) 
				@ [Fill Both, Expand true],
			[Relief Raised,Height 14, Width 14],		
			[Bind("<Button-1>",cmd)])
   end;

fun build_file_list win cont = 
   let fun cmd (_:TkEvent) = 
       let val Mark(n,_) = readCursor "fileliste"
	    val _ = Debug.print 12 ("FILER n= "^(makestring n)^"\n");
	    val file = nth(!FILES,n)
       in  SEL_FILE_DIR := (!CUR_DIR);
	   SEL_FILE_FIL := file;
	   clearText "fileentry";
	   insertTextEnd "fileentry" (file)
       end
       fun cmd1   e = (cmd(e); cd (!EXT_CUR_DIR); closeWindow win;
			      cont(!SEL_FILE_DIR, !SEL_FILE_FIL))	
       fun putCmd ev = M.CB.put (M.fileToObj(!SEL_FILE_DIR, !SEL_FILE_FIL))
		               ev (fn () => (closeWindow win))
   in Listbox("fileliste",RightScb,
			[Side Right, Fill Both, Expand true],
			[Relief Raised,Height 14, Width 14],		
			[Bind("<ButtonPress-1>",cmd),
			 Bind("<ButtonRelease-1>", putCmd),
			 Bind("<Double-Button-1>",cmd1)]) end;

fun boxblock win cont = 
    Frame(newWidgetId(),[build_dir_list win true,build_file_list win cont],
			[Fill Both, Expand true],[],[]);



fun quit_but win cont= 
    let fun cmd () = (cd (!EXT_CUR_DIR);
		      closeWindow win;
		      cont(!SEL_FILE_DIR, !SEL_FILE_FIL))
    in  Button(newWidgetId(),[Fill X,Side Right,Expand true],
				[Text"Quit",Command cmd],[]) 
    end;

fun canc_but win cont= 
    let fun cmd () = (cd (!EXT_CUR_DIR);
			SEL_FILE_DIR := "";
			SEL_FILE_FIL := "";
			closeWindow win;
			cont("",""))
    in Button(newWidgetId(),[Fill X,Side Left,Expand true],
			[Text "Cancel", Command (cmd)],[]) end;

fun bot_line win cont = 
    Frame(newWidgetId(),[quit_but win cont, canc_but win cont],
			[Side Bottom, Fill X],[],[]);


val directory_label = Label(newWidgetId(),[Fill X],[Text "Directory"],[]);
fun directory_entry sw = 
    let fun cmd (_:TkEvent) = 
	let val newdir = readTextAll "direntry"
	in (cd newdir; CUR_DIR:= newdir;
	    if sw then showDirAndFiles() else showDir ())
	    handle NotDirectory => 
		errs_occurred1("Can not access directory: "^newdir)()
	end
    in  Entry("direntry",[Fill X],[],[Bind("<Return>",cmd)]) end;
val filter_label = Label(newWidgetId(),[Fill X],[Text "Filter"],[]);

val filter_entry = 
    let fun cmd (_:TkEvent) = (CUR_FILE_FILTER:= readTextAll "filterentry";
			       showDirAndFiles ())
    in Entry("filterentry",[Fill X],[],[Bind("<Return>",cmd)]) end;

val file_label = Label(newWidgetId(),[Fill X],[Text "File"],[]);
val file_entry = Entry("fileentry",[Fill X],[], 
				[Bind("<Return>",fn _ => ())]);



fun fileselect_win winId cont =
let fun  init_filer_win () = 
    (CUR_DIR:= pwd();
     EXT_CUR_DIR:=(!CUR_DIR);
     showDirAndFiles())
in  (winId,[WinTitle "File Selection"],
	 [directory_label,directory_entry true,filter_label,filter_entry,
	  boxblock winId cont,file_label, file_entry, bot_line winId cont],
     init_filer_win)
end;


fun dirselect_win winId cont = 
let fun  init_filer_win () = 
    (CUR_DIR:= pwd();
     EXT_CUR_DIR:=(!CUR_DIR);
     showDir())
in  (winId,[WinTitle "Directory Selection"],
	 [directory_label,directory_entry false,filter_label,filter_entry,
	  build_dir_list winId false, bot_line winId cont],
     init_filer_win)
end;


fun file_select  ()= 
    (SEL_FILE_DIR := "";
     SEL_FILE_FIL := "";
     startTclExn [fileselect_win "main" (fn (_,_) => ())];
     (!SEL_FILE_DIR, !SEL_FILE_FIL));

fun dir_select  ()= 
    (SEL_FILE_DIR := "";
     SEL_FILE_FIL := "";
     startTclExn [dirselect_win "main" (fn (_,_) => ())];
     (!SEL_FILE_DIR));

fun enter_file cont = 
    (SEL_FILE_DIR := "";
     SEL_FILE_FIL := "";
     openWindow (fileselect_win "filesel" cont));

fun enter_dir cont= 
    (SEL_FILE_DIR := "";
     SEL_FILE_FIL := "";
     openWindow (dirselect_win "dirsel" (fn (a,_) => cont(a)) ));


end  

end;

(* Useful partial instantiations of the filer. *)

(* Instantiate with the regular expression matcher by
 * Ryan Stansifer.
 *)

functor ClipFiler
    (structure M : 
	 sig structure CB: CLIPBOARD_W
	     val fileToObj : string* string-> CB.obj
	     val dirToObj  : string-> CB.obj
	 end): FILER_SIG =

    RegExClipFiler(structure M = 
		       struct
			   exception bad_regexp			   
			   val filter_files = Rex.rex
			       handle Rex.bad_rex => raise bad_regexp
			   structure CB  = M.CB
			   val fileToObj = M.fileToObj
			   val dirToObj  = M.dirToObj
		       end)



(* 
 * Simple Filer without a clipboard, and the default regular expression 
 * matcher 
 *)

structure Filer : FILER_SIG

 = ClipFiler (structure M= 
		  struct      		
		     structure CB     = DummyCB
		     val fileToObj    = fn _ => ()
		     val dirToObj     = fn _ => ()
		  end)




