
(* ***************************************************************************

   $Source: /home/uniform/rep/sml/sml_tk/toolkit/gen_gui.sig,v $

   Signature for the generic graphical user interface. 

   APPL_SIG is the signature for an application with which to instantiate
   gen_gui.

   GENGUI_SIG is the export signature of genu_gui

   See <a href=file:../doc/DOC.html>the documentation</a> for more
   details.  "tests+examples/simplesinst.sml" contains a small example
   of how to use this package.
 
   $Date: 1997/08/25 10:45:31 $
   $Revision: 1.19 $
   $Author: cxl $

   (C) 1996, Universitaet Bremen

  ************************************************************************** *)



(* 
 
 Configuration for the gen_gui-- modfiying its visual appearance
 
 *)

signature GENGUI_CONF =

    sig
	(* This is the width and height of the construction area *)
	val width         : int
	val height        : int       
	    
	(* The background colour of the construction area *) 
	val background    : TkTypes.Color
	    
	(* The font and the width of the box, in pixels,
	 * used to display the icons *)
	val iconNameFont  : TkTypes.Font
	val iconNameWidth : int
	    
	(* The icon used to display the trashcan, and its initial position *)
	(* NB. Can only construct sml_tk icons at runtime *)
	val trashcanIcon  : unit-> Icons.icon
	val trashcanCoord : TkTypes.Coord  (* should disappear soon *)
	    
	(* if opaqueMove is true, then the whole item will move if it
	 * is grabbed; ows. only change cursor to indicate an object is
	 * being moved. *)
	val moveOpaque    : bool

	(* if oneWindow is true, the construction area will appear as 
	 * a widget within the lower part of the window (which will be 
	 * large enough to hold it. Actually, this should be called _one
	 * widget_, since it means that both construction and assembly
	 * come within one frame); otherwise, the construction area 
	 * will appear as a separate window 
	 *)
	val oneWindow     : bool
	    
	(* The height and width of the construction area, and the
	 * position of the window. The X/Y position and the wsTitle options
	 * determine the placing and title of the construction area window,
	 * and only take effect if oneWindow is false. 
	 *)
	val wsHeight      : int
	val wsWidth       : int
	val wsXY          : (int* int) TkTypes.option
	val wsTitle       : string-> string

	(* The minimum distance between two objects' dropzones when placing
	 * new objects
	 *)
	val delta         : int
    end	    
 
 


(*

 Signature for a single application with which the generic GUI can
 be instantiated. 

 *)

signature APPL_SIG =
  sig
    eqtype objtype
    type object
 

    (* New objects are objects together with a specification where
     * they should appear.  This is a coordinate followed by an Anchor
     * which gives the direction in which GenGUI tries to place the
     * object if another object is in the way.  
     *)
    type new_object = object * TkTypes.Coord* TkTypes.AnchorKind


    (* Typing *)

    val obj_type     : object -> objtype

    val objlist_type : object list -> objtype TkTypes.option

    val is_tool      : objtype-> bool  (* objects of this type are tools *)
    val is_primary   : objtype-> bool  (* objects of this type are primary *)

    (* Each object has a name, which can be set: *)

    val sel_name     : object -> string
    val set_name     : object* string -> unit

    (* with the obvious axiom (setName(o,s); selName(o))= s *) 

    (* Icons *)
    val icon         : object -> Icons.icon      
 

    (*
     * Nullary objects are constants, or in other words, objects
     * existiting a priori. 
     * The init function returns a list of all these objects; it will 
     * only be called once, on startup.
     *)
    val init   : unit -> new_object list

    (* Unary operations *)

    (* standard operations *)
    val show   : object -> unit
    val stat   : object -> unit
    val delete : object -> unit
    
    (* further object type specific operations: for a type t, monOps t
     * is a list of pairs (f, s), where f is a unary operation, and s
     * is a string, the name under which it appears in the pop-up
     * menu. f has the functionality 
     *     object* TkTypes.Coord-> (newObject-> unit)-> unit;
     * where the first argument is the object itself, together with its present
     * location, and the second argument is a continuation you can use
     * to create new objects.  
     *) 
    val mon_ops : objtype -> 
	         ((object* TkTypes.Coord -> 
		   (new_object ->  unit) -> unit) * string) list

    (*
     *
     * binary operations 
     *
     * aka.the drag&drop table
     *)
    val bin_ops :  objtype * objtype -> 
	(object* TkTypes.Coord-> object list-> new_object list) TkTypes.option 


    (* --- The Construction Area ----------------------------------------- *) 

    type ca

    (* This data type represents the Construction Area's
     * state. It might eg. probably contain the area's
     * widget's widget id.
     *)
	
    (* This should be the respective row of the drag&drop table in
     * binaryOps above. Has to be here explicitly, since it will change
     * the whole area rather than just the object.
     * Also, objects may behave differently when opened.
     *)	
    val  area_ops  : objtype-> ca-> object list-> unit
	
    (* open an object to be worked on the construction area
     * The old object is deleted from the manipulation area.
     *
     * One (or even more?) new objects may appear on the manipulation
     * area when the construction finishes, they are introduced with 
     * the second argument.
     *
     * The result is a tuple, consisting of a data structure
     * ws as above, a list of widgets representing the 
     * area on the screen and an init function to be called after 
     * the widget has been placed and that would not be necessary
     * if we could instantiate text widgets properly. 
     *)
    val area_open  : TkTypes.WinId* object* (object -> unit) -> 
                                      (ca* TkTypes.Widget list* (unit-> unit))
    val area_init  : unit-> unit
		(* initializations that need to be done only once.
		 * !!! Caution, this is called when the area isn't open.
		 *)

    type objectlist = object list (* only needed 'cause ML disnae like
				   * sharing type obj = object list *)
	
    structure CB : CLIPBOARD
    sharing type CB.obj = objectlist
	    (* This clipboard will allow the exchange of items between 
	     * the drag&drop area and other application-specific 
	     * widgets-- eg. a chooser. 
	     *)
	
	    (* 
	     * configuration -- see above
	     *)
    structure Conf : GENGUI_CONF
 
  end


(*

 Export signature for the generic GUI. 

 *)

signature GENGUI =  
  sig
     type object
     type new_object

     (* the state of the gui *)
     type gui_state = new_object list

     val intro : new_object -> unit 
	 (* introduce (not "create" really) a new object into the
	  * manipulation area
	  *
	  * Note that there is no way to delete objects from outside the
	  * GenGUI. This is a design decision.
	  *)

     (* The GenGUI main widget. You MUST use the init function below
      * to initialize this widget. (Note GenGUI doesn't check this 
      * itself.)
      *)
     val main_wid : TkTypes.WinId -> TkTypes.Widget 

     
     (* In the following, init takes a gui_state and returns a function
      * which has to be used as the init function of the main window,
      * as it sets up the GenGUI. 
      *
      * state returns a gui_state suitable as an argument to init. 
      *)

     val init  : gui_state-> unit
	 (* call that as init action of main window *)

     val state : unit-> gui_state

     (* This is the initial state which only has those objects as given
      * by the application's init() function (see above). 
      *)
     val initial_state : unit-> gui_state

     exception GenGUI of string 
	 (* something went wrong-- this execption indicates a
	  * critical error on part of the gen_gui. This may either
	  * be a genuine bug (although due to the state-of-the-art
	  * software technology used to implement gen_gui, this is un-
	  * likely), or wrong usage of GenGUI. 
	  * The exception is critical in the sense that it's all right
	  * to just panic after it has been raised. Alternatively, catch
	  * it, ignore it and hope for the best. 
	  *)

     type objectlist= object list 

     structure CB : CLIPBOARD
     sharing type CB.obj = objectlist
	 (* This clipboard will allow the exchange of items between 
	  * the drag&drop area and other application-specific 
	  * widgets-- eg. a chooser. 
	  *)

  end


(* 
functor GenGUI(structure appl: APPL_SIG ) :
  sig 
      include GENGUI_SIG 
      sharing type object     = appl.object 
	  and type new_object = appl.new_object
  end

= ? 

*)




