(* ***************************************************************************

   $Source: /home/uniform/rep/sml/sml_tk/toolkit/gen_gui.sml,v $

   A generic graphical user interface. 
  
   See <a href=file:../doc/DOC.html>the documentation</a> for more
   details.  

   "tests+examples/simplesinst.sml" contains a small example of how to
   use this package.
 
   $Date: 1997/08/25 10:45:37 $
   $Revision: 1.32 $
   Author: cxl (Last modification $Author: cxl $)

   (C) 1996, Bremen Institute for Safe Systems, Universitaet Bremen

  ************************************************************************** *)




functor GenGUI(structure appl: APPL_SIG ) :
  sig 
      include GENGUI
      sharing type object    = appl.object
	  and type new_object = appl.new_object
  end

= 

struct 
    
    open TkTypes SmlTk BasicUtil 

(* ************************************************************************ *
 * 
 * Parameters 
 *
 *)

    (* 
     * The trashcan
     *)
    val trashcanCId      = mkWidgetId("trashcan") 

    fun trashcanCItem () =
	(* the trashcan does _not_ have an entry you can change *)
	CIcon(trashcanCId, appl.Conf.trashcanCoord, 
	      Icons.selImage (appl.Conf.trashcanIcon()),
	       [Anchor NorthWest], [])	       

    datatype item = obj of WinId* WidId * CItemId * Rect * appl.object
                  | trashcan of Rect

    (* the workspace frame widget id *)
    val wsFrameId        = mkWidgetId("ws")
    (* the widget id of the canvas all the items are placed on *)
    val backdropId       = mkWidgetId("backdrop")

   (*  
    * Assorted global references (ah, the joys of functional programming). 
    *)
    val placeObj = ref (fn (c:WidId)=> fn (i:item)=> ())
    val delObj   = ref (fn (c:WidId)=> fn (i:item)=> (fn ()=> ()))
    val overDZ   = ref (fn (c:WidId)=> fn (c:int* int)=> ([]:item list))
    (* point to the functions to place and delete items on the d/d canvas,
     * and to check dropzones as exported by the drag&drop module. 
     *)

    exception GenGUI of string
		
    fun isStdTool (trashcan _)    = true
      | isStdTool _               = false

    (* selector functions *)

    fun selCanvas  (obj x)        = #2 x
      | selCanvas  (trashcan _)   = backdropId
	
    fun selDropZone (obj x)       = #4 x
      | selDropZone (trashcan dz) = dz

    fun selItemId (obj x)         = #3 x
      | selItemId (trashcan _)    = trashcanCId

    fun selObj     (obj x)        = #5 x

    fun itemCoords oo = hd (readCItemCoords (selCanvas oo) (selItemId oo))

    fun bitmapCId   str = subWidId(str, "xIcon")
    fun widgetCId   str = subWidId(str, "xWidId")
    fun entryWId    str = subWidId(str, "xEntry")
    fun popUpMenuId str = subWidId(str, "xKuckuck")



    fun setObjImg  which ca cit oo = 
	                     addCItemConf ca (bitmapCId cit)
			           [Icon (which (appl.icon oo))]

    fun setItemImg which (obj(_, ca, cit, _, oo)) = 
	                     setObjImg which ca cit oo
      | setItemImg which (trashcan bollocks) =
                             addCItemConf backdropId trashcanCId 
                                   [Icon (which (appl.Conf.trashcanIcon()))]

    val hiliteIcon  = setItemImg Icons.selHiLite
    val resetIcon   = setItemImg Icons.selImage
    val outlineIcon = setItemImg Icons.selOutline


    fun debugmsg msg = Debug.print 11 ("GenGUI: "^msg)

    fun anchorToDir North     = (0, ~1)
      | anchorToDir NorthEast = (1, ~1)
      | anchorToDir East      = (1, 0)
      | anchorToDir SouthEast = (1, 1)
      | anchorToDir South     = (0, 1)
      | anchorToDir SouthWest = (~1,1)
      | anchorToDir West      = (~1, 0)
      | anchorToDir NorthWest = (~1, ~1)
      | anchorToDir Center    = (0, 0)

    (*
     * Find a place to put the new object
     * 
     * Currently, this function just wanders off into the direction given until
     * it either finds a free space or wanders off the canvas. 
     * It would be nice if it would be a bit more clever, and eg. if it at 
     * first can't find something in Direction NE, first try N, then E, and
     * then go further NE etc.
     *)

    fun getDropZone icn (x, y)= 
	((x, y), (x+Icons.selWidth icn, y+Icons.selHeight icn))
                        (* the drop zone is always in relation to
			 * the _first_ sub-item, here the bitmap *)

    fun findNicePlace cnv (nuOb, wh, shift) =
	let exception Off
	    (* width and height of canvas *)	    
	    val (cw, ch)= (getWidth cnv, getHeight cnv)
	    fun offCanvas (x, y) = (x<0) orelse (y<0) orelse
		                   (x>cw) orelse (y>ch)
            (* get dropzone of new object *)
	    val dz      = getDropZone(appl.icon nuOb)

	    (* check for another dropzone *)
	    fun noOtherDZ ((x1, y1),(x2,y2)) =
		(null ((!overDZ) cnv (x1, y1))) andalso
		(null ((!overDZ) cnv (x1, y2))) andalso
		(null ((!overDZ) cnv (x2, y1))) andalso
		(null ((!overDZ) cnv (x2, y2)))

	    fun placeIt whr Center = whr
	      | placeIt whr sh     =
		    if (offCanvas whr) then raise Off
		    else if noOtherDZ (dz whr) then whr
			 else (debugmsg ("Can't place at "^(Coord.show [whr]));
			       placeIt ((addCoord whr) 
				       (smultCoord (anchorToDir sh) 
					appl.Conf.delta)) sh
			      )
	in
	    placeIt wh shift handle Off => wh
	end

    fun openWorkspace win bd cid dz oo ev =
	let
	    val wsWidIds = ref ([]: WidId list)
	    val where = hd (readCItemCoords bd cid)
	    (* bindings for the workspace while open *)
	    fun wsEnter wsp ev =
		let val dropobs = appl.CB.copy ev
		    val oot      = appl.objlist_type dropobs
		in  case oot of 
		    SOME ot => appl.area_ops ot wsp dropobs
		  | NONE => () 
		end
	        handle appl.CB.Empty => ()
	    fun wsBindings wsp = 
		[Bind("<Enter>", wsEnter wsp)]
	    (* bindings for the workspace while closed *)
	    val wsClosedBindings =
		[Bind("<Enter>", fn _ => ())]
	    fun ridItem deleteOldItem nuOb =
		((* kindly disregard the following unpleasantries- thank you *)
		 deleteOldItem();
		 placeOnAreaAt win bd where nuOb;
		 (* Delete citem of old object (the dditem has been deleted 
		  * when opening it), and place the new object. 
		  * Bit of a hack as well, we actually just want to replace
		  * the object. This would mean a change in the interface
		  * of drag_and_drop. I'll think about it.
		  *)
		 Sema.V()
		)
	    fun closeWid delItem nuOb = 
		(app delWidget (!wsWidIds); ridItem delItem nuOb)
	    fun closeWin win delItem nuOb =
		(closeWindow win; ridItem delItem nuOb)
	in
	    if (appl.is_primary (appl.obj_type oo)) andalso (Sema.P()) then
		 let val deleteOldItem =
		          (setObjImg Icons.selOutline bd cid oo;
		                        (* outline corresponding object *)
			   (!delObj) bd (obj(win, bd, cid, dz, oo))
			                (* delete item from d&d canvas,
					 * returns closure to delete item *)
			   )
		 in
		     if appl.Conf.oneWindow then
			 let val (wsp, wwidgs, init) =
			     appl.area_open(win, oo, closeWid deleteOldItem)
			 in
			     (wsWidIds := map selWidgetId wwidgs;	      
			      app (addWidget win wsFrameId) wwidgs;
			      addBind wsFrameId (wsBindings wsp);
			      init()
			      )
			 end
		     else
			 let val winId = newWinId()
			     val (wsp, wwidgs, init) =
				 appl.area_open(win, oo, 
						closeWin winId deleteOldItem)			     fun addWSBind widg = addBind (selWidgetId widg) 
				 (wsBindings wsp) 
			 in  (openWindow(winId, 
				[WinTitle (appl.Conf.wsTitle (appl.sel_name oo)),
				 WinGeometry (NONE, appl.Conf.wsXY),
				 WinGroup win],
				      wwidgs, init);
			      (* add binding to _all_ widgets: *)
			      app addWSBind wwidgs
			      )
			 end
		 end
	    else
		debugmsg "Not a primary object, or ConArea already open."
     end
		     
    and monOpMenu win cnv cid dz ob =
	let (* standard operations menue *)
	    fun opStdMItem (opn, name) =
		   MCommand [Text name, Command (fn () => opn ob)]
	    fun stdOps cnv cid dz 
		= [(appl.show, "Show"),
		   (appl.stat, "Show statistics"),
		   (fn ob=> (!(delObj) cnv (obj(win, cnv, cid, dz, ob)) ();
		             appl.delete ob), "Delete")]
	    (* customized extra menue *)
	    val obt        = appl.obj_type ob
	    fun opMenuItem ob (opn, name) =
	           MCommand [Text name, 
			     Command (fn () => opn (ob, 
						    hd (readCItemCoords cnv cid))
				                   (placeOnArea win cnv))]
	    val moreOpList = map (opMenuItem ob) (appl.mon_ops obt)
	    val menuList   = (map opStdMItem (stdOps cnv cid dz)) @
		                   (if (null moreOpList) then []
			            else MSeparator :: moreOpList)
        in  Popup(popUpMenuId cid, false, menuList)
	end

    and popUpMonOpMenu cid (TkEvent(_,_,_,_, xr, yr)) =
	 popUpMenu (popUpMenuId cid) (SOME 0) (xr, yr)
	 
    and objectBindings win cnv cid dz ob =
	[Bind("<Double-Button-1>", openWorkspace win cnv cid dz ob),
	 Bind("<Button-3>", popUpMonOpMenu cid)
	 ]
		
    and textEntryBindings win ob cid =
	[Bind("<Return>", 
	      fn _ => (appl.set_name(ob, readTextAll (entryWId cid));
	               deFocus win (* was backdrop ??? *)))]
(*
    and delObjItem cnv id =
	app delCItem [bitmapCId id, widgetCId id, id]
 *)

    and placeObjAsItem win cnv (x,y) nuOb =
	let val cid   = newCItemId()
	    val icn   = appl.icon nuOb
	    val bm_w  = Icons.selWidth icn
	    val bm_h  = Icons.selHeight icn
	    val te_x  = (bm_w- appl.Conf.iconNameWidth) div 2
	    val dz    = getDropZone icn (0, 0)

	    (* the CItems representing the object *)
	    val bmCI  = CIcon(bitmapCId cid , (x, y), Icons.selImage icn,
			      [Anchor NorthWest], 
			       objectBindings win cnv cid dz nuOb)
	    val teCI  = CWidget(widgetCId cid, (x+te_x, y+bm_h),
			       newCItemFrameId(),
                               [Entry(entryWId cid, [Expand true], 
				      [Font appl.Conf.iconNameFont],
				      textEntryBindings win nuOb cid),
			        monOpMenu win cnv cid dz nuOb],
			       [], [Width appl.Conf.iconNameWidth,
				    Anchor NorthWest],
                               (objectBindings win cnv cid dz nuOb))
	    val tagCI = CTag(cid,  [bitmapCId cid, widgetCId cid])	
	in  (app (addCItem cnv) [bmCI, teCI, tagCI];
	     clearText (entryWId cid);
	     insertTextEnd (entryWId cid) (appl.sel_name nuOb);
	     obj(win, cnv, cid, dz, nuOb))
	end

    and placeOnAreaAt win cnv where nuOb = 
	   (!placeObj) cnv (placeObjAsItem win cnv where nuOb)

    and placeOnArea win cnv (nuOb, wh, shift) = 
           placeOnAreaAt win cnv (findNicePlace cnv (nuOb, wh, shift)) nuOb



    
    structure DD=
	struct
	    type item = item
	    type il   = item list
		
	    val selItemId = selItemId
	    val selDropZone  = selDropZone
		
	    fun isImmobile _ = false
		
	    fun grab it = 
		if not appl.Conf.moveOpaque then
		    outlineIcon it
		else ()
		    
	    val release  = resetIcon
		
	    val select   = hiliteIcon 
		
	    val deselect = resetIcon
		
	    fun move it delta =
		if  appl.Conf.moveOpaque then 
		    moveCItem (selCanvas it) (selItemId it) delta
		else ()
		    
	    fun enter eIt entering =
		if (List.exists isStdTool entering) orelse
		    (List.exists (appl.is_tool o appl.obj_type o selObj) entering) 
		    then false
		else
		    case eIt of
			trashcan d =>  (hiliteIcon (trashcan d); true)
		      | obj(_, _, cit, dz, ob)=>
			    let val olt= appl.objlist_type (map selObj entering)
				val ot= appl.obj_type ob
			    in  case olt of
				NONE    => false
			      | SOME lt =>
				    (case appl.bin_ops (ot, lt) of
					 NONE    => false
				       | SOME f  => (hiliteIcon eIt; true))
			    end
			
	    val leave = resetIcon
		
		    
	    fun drop (trashcan _) trash =       
		(app appl.delete (map selObj trash); false)
	      | drop (obj(win, cnv, cid, dz, ob)) dropped =
		let val olt  = appl.objlist_type (map selObj dropped)
		in  case olt of
		    NONE    => raise GenGUI "Illegal 'drop'"
		  | SOME lt =>
			(case appl.bin_ops(appl.obj_type ob, lt) of
			     NONE    => raise GenGUI "Illegal 'drop'"
			   | SOME f  => let val nuObjs = f (ob, hd (readCItemCoords cnv cid))
				 (map selObj dropped)
					in  (app (placeOnArea win cnv) nuObjs; true)
					end)
		end
	    
	
	
	    structure CB = (* Clipboard(struct type obj= item end) *)
		struct
		    type obj= item list
			
		    fun  put it ev cb = appl.CB.put (map selObj it) ev cb
		end
	end
    
    structure ObjectsDD= DragAndDrop(DD)


    type object     = appl.object
    type objectlist = appl.objectlist
    type new_object = appl.new_object
    type gui_state  = new_object list 

    (* the clipboard *)
	
    structure CB= appl.CB;

    fun enterArea win bd (ev as TkEvent(_,_, x, y, _, _)) =
	app ((ObjectsDD.place bd)o (placeObjAsItem win bd (x,y))) (CB.get ev)
	handle CB.Empty => ()

    fun areaBindings win bd =
	[Bind("<Enter>", enterArea win bd)]
 
    fun placeStdTools bd =
	let val tci = appl.Conf.trashcanIcon()
	    val tcw = Icons.selWidth tci
	    val tch = Icons.selHeight tci
	in  (addCItem bd (trashcanCItem());
	     ObjectsDD.place bd (trashcan ((0,0), (tcw, tch)))
	     )
	end
 

    fun initRefs () = 
	(placeObj := (fn cnv=> ObjectsDD.place cnv);
	 delObj   := (fn cnv=> ObjectsDD.delete cnv);
	 overDZ   := (fn cnv=> ObjectsDD.overDropZone cnv);
	 Sema.V()
	 ;debugmsg "Refs init'd."
	 )
	
    val backdropWinId = ref (mainWinId())
	
    fun main_wid win =
	let val assArea= Canvas(backdropId, NoneScb, [],
				[Expand true, Fill X, Fill Y, Side Top],
				[Width appl.Conf.width, Height appl.Conf.height, 
				 Relief Groove, Background appl.Conf.background], 
				areaBindings win backdropId)
	    val _ = backdropWinId := win
	in if appl.Conf.oneWindow then
	    Frame(newWidgetId(),
		  [assArea,
		   Frame(wsFrameId, [], [Side Bottom],
			 [Relief Groove, Width appl.Conf.wsWidth,
			  Height appl.Conf.wsHeight],
			 [])
		   ], [], [], [])
	   else
	       assArea
	end


    val initial_state = appl.init
	
	
    fun init state =
	let val bd= ObjectsDD.init backdropId
	in  (initRefs();
	     placeStdTools bd;
	     app (placeOnArea (!backdropWinId) bd) state;
	     appl.area_init()
	    )
	    handle ObjectsDD.DragAndDrop error => 
		                  raise GenGUI ("D/D error:"^error)
	end

    (* some thought needed here. If we ever want to place objects in
     * areas other than the backdrop, we need to pass this the WinId and WidId 
     * of the respective canvas. This becomes vital once we start exporting
     * configurations with more than one canvas -- i.e. folders. 
     *)

    fun intro nuOb = placeOnArea (!backdropWinId) backdropId nuOb
    
    fun state ()   = 
	let fun item2nuObj (obj(win, wid, cid, dz, ob))=
	                         SOME (ob, hd (readCItemCoords wid cid)
				           handle Empty=> (0, 0), Center)
	      | item2nuObj (trashcan _) = NONE
	    
	in  List.mapPartial item2nuObj (ObjectsDD.allItems backdropId)
	    (* This just forgets abouts the trashcan-- so it always appears
	     * in the same position on startup. One may or may
	     * not want to change that. 
	     *)
	end

end



