(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/toolkit/util_win.sig,v $
 
   Utitility windows -- signature file.
  
   Windows for errors, warnings, user confirmation and text entry. 

   Supplements, replaces and extends err_warn_windows.sml and
   enter_windows.sml. 

   $Date: 1997/04/06 19:06:01 $
   $Revision: 1.8 $
   Author: cxl (Last modification by $Author: cxl $)$Author: cxl $

   (C) 1997, Bremen Institute for Safe Systems, Universitaet Bremen
 
  ************************************************************************** *)

(* export signature *)
signature UTIL_WIN =
sig
    (* There is no clear distinction between an "error" and a "warning";
     * the intention is that a warning indicates that something didn't suceed
     * whereas an error indicated that something went wrong.
     *)

    (* display error or warning, then continue *)    
    val error   : string -> (unit-> unit)-> unit
    val warning : string -> (unit-> unit)-> unit

    (* Demand confirmation, then execute continuation.
     * If user clicks "cancel", do nothing *)
    val confirm : string -> (unit-> unit)->unit
	
    (* display a text using a given width and, perhaps, a font 
     *
     * nb. the first parameter is the window's title, the second the width,
     * the third, the font (mibbe), the fourth the text to
     * be displayed, and the last a continuation which is passed the
     * window id for use in further redisplays;
     * redisplay displays a string in a previously openend window. If WinId
     * doesn't exists or isn't a window opened with display, you'll get 
     * an exception (it should check and raise a specific one...)
     *)
    val display  : string-> int-> TkTypes.Font TkTypes.option-> string->
                                            (TkTypes.WinId-> unit)-> unit
    val redisplay : TkTypes.WinId-> string-> unit

    (* prompt the user to enter a text in a separate window w/ a text widget
     *
     * The first three parameters are the title of the window, the prompt for
     * the text and the default value for the text itself; the next two
     * parameters are the width/height of the textwidget (in chars);
     * then the continuation to be called with the entered text, and
     * a continuation to be called when the user aborts the text entry
     *)
    val enterText : string-> string-> string-> int-> int-> 
	                        (string-> unit)-> (unit-> unit)-> unit


    (* prompt the user to enter a line of text in a separate windw 
     *
     * Parameters are as before (but no height). This function uses an 
     * artificially intelligent semi-heuristic fuzzy logic based algorithm 
     * implemented in Java to determine wether the text entry should be 
     * alongside the prompt or below it.
     *)
    val enterLine : string-> string-> int-> 
	                        (string-> unit)-> (unit-> unit)-> unit

    (* (Actually, if the prompt is at least twice as long as the text entry,
     *  it is below the prompt, otherwise to its right). *)



end

(* argument signature contains configuration options *)
signature UTILWIN_CONFIG =
sig
    (* width and font for the messages *)
    val msgWidth           : int
    val msgFont            : TkTypes.Font

    (* width and font for "OK"/"Cancel" buttons *)
    val buttonWidth        : int
    val buttonFont         : TkTypes.Font

    (* paths to the "error" and "warning" icons/gifs *) 
    val errorIconFilenm   : unit-> string
    val warningIconFilenm : unit-> string

    (* font used for text to be entered *)
    val enterTextFont     : TkTypes.Font

end

(* functor UtilWin (config: UTILWIN_CONFIG) : UTIL_WIN = ? *)
