(* ***************************************************************************
 
   $Source: /home/uniform/rep/sml/sml_tk/toolkit/util_win.sml,v $

   Utitility windows.
  
   Windows for errors, warnings, user confirmation and text entry. 

   Supplements, replaces and extends err_warn_windows.sml and
   enter_windows.sml. 

   $Date: 1997/07/27 21:53:58 $
   $Revision: 1.12 $
   $Author: cxl $
 
  ************************************************************************** *)


functor UtilWin (config: UTILWIN_CONFIG) : UTIL_WIN =

struct

    local open BasicTypes BasicUtil SmlTk in


    (* parameter section *)

    val avgLetterSize = 12 
        (* average size of a letter in pixels, needed because Tcl/Tk is
	 * too bloody stupid. Not sure if this should be part of config.
	 *)

    val buttonRelief = Raised
	(* relief of the ok/cancel buttons *)

    
    fun errwrnwidgs iconpath msg cc = 
	[Label(newWidgetId(), [Side Left, Fill Y],
	       [Icon (FileImage (iconpath, newWidgetId()))], 
	       []),
	 Frame(newWidgetId(),
	      [Message(newWidgetId(),[Side Top, Fill X],
				   [Text msg, 
				    Width config.msgWidth,
				    Font config.msgFont],[]),
	       Button(newWidgetId(),[Side Right],
				    [Text "Continue", Command cc, 
				     Relief buttonRelief,
				     Width config.buttonWidth,
				     Font config.buttonFont], [])
	      ], [Side Top, Fill Both], [], [])
	]
	 

   fun errwrnwin title iconpath msg cc =
       let val wid    = newWidgetId()
	   val close  = cc o (P closeWindow wid)
       in
	   (wid, [WinTitle title, WinTransient NONE], 
	    errwrnwidgs iconpath msg close, K())
       end
		
		
   fun error   msg cc = openWindow (errwrnwin "Error Message" 
				    (config.errorIconFilenm()) msg cc)

   fun warning msg cc = openWindow (errwrnwin "Warning Message" 
				    (config.warningIconFilenm()) msg cc)



   (* --- Confirmation [ OK ]  [ Cancel ] -------------------------------- *)


   fun buttonConf () = [Width  config.buttonWidth,
			Relief buttonRelief,
			Font   config.buttonFont]


   fun OkCancelButtons win cont abort =
       let fun cc () = (closeWindow win; cont())
	   fun no () = (closeWindow win; abort())
       in  Frame(newWidgetId(), 
		 [Button(newWidgetId(), 
			 [Side Left], 
			 [Text "Cancel", Command no]@
			 (buttonConf()), []),
		  Button(newWidgetId(),
			 [Side Right], 
			 [Text "OK", Command cc]@
		  (buttonConf()), [])
		 ], [Side Bottom, Fill X], [], [])
       end


   fun confirm msg cc =
       let val wid = newWidgetId()
           val pic = Label(newWidgetId(), [Side Left, Fill Y],
			      [Icon (FileImage(config.warningIconFilenm(),
					       newWidgetId()))], [])
	   val frm = Frame(newWidgetId(),
			   [Message(newWidgetId(),[Side Top, Fill Both],
				    [Text msg, 
				     Width config.msgWidth,
				     Font  config.msgFont],[]),
			    OkCancelButtons wid cc (K())], 
			   [Side Top, Fill Both], [], [])
       in openWindow (wid, [WinTitle "Please Confirm Or Abort",
			    WinTransient NONE], [pic, frm], K())
       end


   (* --- display a text -------------------------------------------------- *)

   fun textWidId win = win^"xText"

   fun display title w fontspec msg cc =
       let fun quitBut win =  Frame(win^"xFrm",
                                    [Button(win^"xButton", [Side Right],
                                    [Text "Close", 
                                     Command (fn () => closeWindow win)]@
				      (buttonConf()), [])
                                     ], [Side Bottom, Fill X], [], []);
           fun showText win = Message(textWidId win, [Side Top, Fill Both],
					  [Text msg, Borderwidth 10, Width w]@
					  (case fontspec of SOME f => [Font f]
					                  | NONE   => []),
					       [])
           val wid = newWidgetId()
       in  openWindow(wid, [WinTitle title], [quitBut wid, showText wid], fn ()=> cc wid)
       end

   fun redisplay win msg =
       (addConf (textWidId win) [Text msg])



   (* ---- enter windows ------------------------------------ *)
 
   fun buttons win twid cont abort = 
       (* This and OkCancelButtons above are remarkably similar, but
	* I don't know how to implement this in terms of OkCancelButtons *)
       let fun cc () =  let val txt= readTextAll twid
			in  (closeWindow win; cont txt)
			end
	   fun no () =  (closeWindow win; abort())
       in  Frame(newWidgetId(), 
		 [Button(newWidgetId(), 
			 [Side Left], 
			 [Text "Cancel", Command no]@
			 (buttonConf()), []),
		  Button(newWidgetId(),
			 [Side Right], 
			 [Text "OK", Command cc]@
		  (buttonConf()), [])
		 ], [Side Bottom, Fill X], [], [])
       end
	   
   fun enterText title prompt text w h cont abort = 
       let
	   val win     = newWidgetId()
	   val twid    = newWidgetId()
	   val prmpt   = Message(newWidgetId(),
			       [Side Top, Fill X, Expand true], 
			       [Text prompt, Width (w*avgLetterSize)], [])	    
	   val ewtxt   = TextWid(twid, RightScb, text, [], 
			       [Side Bottom, Expand true],
			       [Relief Raised, Borderwidth 2, Width w, Height h,
				Font config.enterTextFont],
			       [])
	   fun init ()  = ()
       in openWindow(win, [WinTitle title], 
		      [buttons win twid cont abort,  ewtxt, prmpt], init)
       end


   fun enterLine title prompt w cont abort =
       let
	   val win   = newWidgetId()
	   val twid  = newWidgetId()
	   val where = if (size prompt) > (w* 2) 
		       then [Side Left, Fill Y, Expand true]
		       else [Side Top, Fill X, Expand true]
	   val prmpt = Message(newWidgetId(),
			       where, [Text prompt], [])
	   val entln = Entry(twid, [Side Left], 
			     [Relief Ridge, Width w, Borderwidth 2, 
			      Font config.enterTextFont],
			     [Bind("<Return>", 
				   fn _ => let val txt= readTextAll twid
					   in  (closeWindow win; 
						cont txt)
					   end)])
       in openWindow(win, [WinTitle title],
		     [Frame(newWidgetId(),[prmpt, entln], [Side Top, Fill X],
			    [], []), buttons win twid cont abort], K())
       end

    end


end


