(* ***********************************************************************

   Project: sml/Tk: an Tk Toolkit for sml
   Author: Stefan Westmeier, University of Bremen
  $Date: 1997/03/26 16:16:36 $
  $Revision: 1.13 $
   Purpose of this file: Operations on Widgets Contents

   *********************************************************************** *)

structure WidgetOps : WIDGET_OPS = 
struct

infix 1 ELEM;
infix 1 /=;
nonfix prefix;

local open BasicTypes in

fun readTextWidState wid =
    let 
	val widg = WidgetTree.getWidgetGUI wid
	val tp   = (Paths.getTclPathGUI o Paths.getIntPathGUI) wid

    in
	if ( (selWidgetWidgetType widg) = Tex ) then
	    case Com.readTclVal (tp ^ ".txt cget -state") of
		"normal"   => false (* TextWidStateNormal *)
	      | "disabled" => true  (* TextWidStateDisabled *)
	else
	    raise WIDGET "WidgetOps.readTextWidState: applied to non-TextWidget"
    end

fun setTextWidReadOnly wid st =
    let 
	val widg = WidgetTree.getWidgetGUI wid
	val tp   = (Paths.getTclPathGUI o Paths.getIntPathGUI) wid
    in
	if ( (selWidgetWidgetType widg) = Tex ) then
	    Com.putTclCmd (tp ^ ".txt configure -state " ^ (Config.showTextWidState st))
	else
	    raise WIDGET "WidgetOps.setTextWidState: applied to non-TextWidget"
    end
    

fun clearTextWidText wid =
    let
	val widg  = WidgetTree.getWidgetGUI wid
	val anl   = Annotation.selTextWidAnnotations widg
	val oldSt = readTextWidState wid
    in
	(setTextWidReadOnly wid false;
	 BasicUtil.app (fn an => AnnotationTree.delete wid (Annotation.selAnnotationId an)) anl;
	 WidgetTree.clearText wid;
	 setTextWidReadOnly wid oldSt)
    end

fun replaceTextWidText wid t ans =
    let
	val oldSt = readTextWidState wid
    in
	(clearTextWidText wid;
	 setTextWidReadOnly wid false;
	 WidgetTree.insertTextEnd wid t;
	 BasicUtil.app (fn an => AnnotationTree.add wid an) ans;
	 setTextWidReadOnly wid oldSt)
    end


(* selectText has to look for the actual text in case of an 
   Entry- TextWid or Listbox widget *)
fun selectText wid (m1,m2) = 
    let 
	val gvf = Com.readTclVal
	val ip =  Paths.getIntPathGUI wid;
        val w  =  WidgetTree.getWidgetGUIPath ip
    in 
	case w of
	    Entry(_,_,_,_)   => 
		gvf((Paths.getTclPathGUI ip) ^ " get")
	  (* Absolute Notloesung. Unklar wie man Selektionen findet !!! *)
	  | TextWid _ => 
		(Com.putLine (Com.writeMToTcl^" ["^(Paths.getTclPathGUI ip) ^ 
			      ".txt get "^(Mark.show m1)^ " " ^
			      (Mark.show m2)^"]") ;
		 Com.getLineM())
	  | Listbox _ => 
		let 
		    val (mt1,_)=((BasicUtil.break SysDep.isDot)o explode)(Mark.show m1);
		    val (mt2,_)=((BasicUtil.break SysDep.isDot)o explode) (Mark.show m2)
		in  
		    gvf((Paths.getTclPathGUI ip)^".box get "^(implode mt1)^" "^(implode mt2))
		end
	  | _  => Config.selText w
    end;

fun selectTextAll wid = selectText wid (Mark(0,0),MarkEnd);


fun selectSelRange wid =
    let 
	val gvf = Com.readTclVal
	val ip =  Paths.getIntPathGUI wid;
        val w  =  WidgetTree.getWidgetGUIPath ip;
        fun G S =((BasicUtil.break SysDep.isDot)o explode)S;
	fun F (m1,m2) = Mark(SysDep.numval m1,SysDep.numval (tl m2));
	fun group []  = [] 
	  | group (a::b::S) = (a,b)::group(S) 
    in 
	case w of
	    TextWid _ => 
		let
		    val s = gvf((Paths.getTclPathGUI ip)^".txt tag ranges sel")
		in
		    group(map (F o G)  (BasicUtil.words(s)))
		end
	  | Listbox _ => 
		 let 
		     val t = gvf((Paths.getTclPathGUI ip)^".box curselection")
		 in  
		     if t="" then 
			 []
		     else 
			 [(Mark(SysDep.numval (explode t),0),MarkEnd)]
		 end
          (* Entry ????????? *)
	  | _            => []
    end;


fun selectSelWindow () = 
    let 
	val gvf = Com.readTclVal
	val t   = gvf("selection own")
    in  
	if t = "" then NONE else SOME (Paths.getIntPathFromTclPathGUI t)
    end;


(* selectCursor has to look for the actual cursor position in listboxes  *)
(* and textwidgets *)

fun selectCursor wid =
    let 
	val gvf = Com.readTclVal
	val ip =  Paths.getIntPathGUI wid;
        val w  =  WidgetTree.getWidgetGUIPath ip
    in 
	case w of
	    TextWid _ => 
		 let 
		     val t = gvf((Paths.getTclPathGUI ip)^".txt index insert")
		     val (m1,m2) =((BasicUtil.break SysDep.isDot) o explode)(t)
		 in 
		     Mark(SysDep.numval m1,SysDep.numval (tl m2)) 
		 end
	  | Listbox _ =>
		 let
		     val t = gvf((Paths.getTclPathGUI ip)^".box curselection")
(*		     val _ = Debug.print 2 ("SelectCursor: t= >"^t^"<") *)
		 in
		     if ( t = "" ) then
			 raise WIDGET "WidgetOps.selectCursor: no selection"
		     else
			 Mark(BasicUtil.atoi t,0)
		 end
          (* Entry ????????? *)
	  | _            => 
		Mark(0,0)
    end;


(* No check that this variable is really defined!!! *)

fun selectVarValue var = 
    Com.readTclVal("global "^var^"; set "^ var)


fun setVarValue var value = 

    (Com.readTclVal("global "^var^"; set " ^ var ^ " " ^ value);
     ())


end 

end
